<# ***************************************************************
Name:		Enable-dvnArchiveMailbox
Version:	1.0.01.15.2015
Author:     Jeff Brown

 Version					Author
 1.0.07142014				Jeff Brown
	Original code adapted from the W7EnableArchiveMailbox.ps1 created by Clifton Leonard
	Notable Changes:
		-Removed numConvert, verifyQuotas, updateQuotas functions - now just set the quotas
		 without checking for previous values
		-Removed writing to a log file - now just write to screen
		-Use Get-MailboxDatabase with filters to find specific versions of archive mailboxes
		 instead of hard coding a list of all the databases
		-If Exchange PSSession is not found, it will launch and prompt for credentials instead
		 of exiting script

 1.0.1072014			    Jeff Brown
	Added color coding for success, failed, and info messages
	Added line breaks to separate when doing bulk archive creations
	Added check at end of script to remove PSSession if it was created

 1.0.01.15.2015             Jeff Brown
    Update to new Devon Function naming convention
    Changed PS Exchange connection test to not try and establish one
    if it's not available
****************************************************************** #>

Function Enable-dvnArchiveMailbox()
{

<#
.SYNOPSIS
	Creates Exchange archive mailbox and sets archive quota to 25GB

.PARAMETER MailboxName
	MailboxName can either be the alias or email address of the Mailbox
	This parameter is optional, the script will prompt for value if not provided

.EXAMPLE
	dvn.Enable-ArchiveMailbox
	This command will prompt for the mailbox to enable with an archive mailbox
	
.EXAMPLE
	dvn.Enable-ArchiveMailbox -MailboxName John.Smith@dvn.com
	This command will enable 'John.Smith@dvn.com' with an archive mailbox
	The mailbox name can either be the email address or the alias of the user

.NOTES
	Use of this function requires Exchange permissions to the following cmdlets:
	Get-User, Get-Mailbox, Get-MailboxDatabase, Enable-Mailbox
	
	Version 1.0.1072014
#>

#**********   Parameters   **********

param(
	[Parameter(Position=0, Mandatory=$false)]
	$MailboxName
)

#**********   Variables   **********

# Mailbox Quotas (some may be temporary)
$IssueWarningQuota 			= 50GB 
$ProhibitSendQuota 			= 50GB
$ArchiveWarningQuota 		= 20GB 
$ArchiveQuota 				= 25GB
$ManagedFolderMailboxPolicy = $Null
$ProhibitSendReceiveQuota 	= "unlimited"	# Shouldn't be adjusted during archive process
$PowerShellVirtualDirectory	= "http://odcmsmxex201p.corp.dvn.com/PowerShell"
$EnableArchiveMB			= $false
$AllMBInfo					= ""
$DomainController			= ""
$errorText					= "white"
$errorBackground			= "darkred"
$success					= "green"
$info						= "yellow"

# Default stature for errors returned when executing remote commands
$EAPreference = "SilentlyContinue"

# Default domain to query/execute
$domain = "corp.dvn.com"

#**********   Main Code   **********

Write-Host

# Verify PS Connection to Exchange
$connExchange = $False	
$PSSessions = Get-PSSession | select ConfigurationName,Availability
foreach ($PSSession in $PSSessions){
	If ($PSSession.ConfigurationName -ilike "Microsoft.Exchange")
    {
		$connExchange = $True
	}
}

# If no PSSession found for Exchange, exit function
If ($connExchange -eq $False)
{
	Write-Host "`tERROR: Current Remote PS Session to Exchange NOT Detected" -ForegroundColor $errorText -BackgroundColor $errorBackground
	Write-Host "`tPlease create remote PS Session to Exchange before continuing" -ForegroundColor $info
    Write-Host "`tExiting Script..." -ForegroundColor $info
    RETURN
}

# Loop until user enters name
If (($MailboxName -eq "") -or ($MailboxName -eq $Null)){
	Do {$MailboxName = Read-Host "`tEnter name of mailbox to archive enable (eg, 'JohnD', 'John.Doe@dvn.com')"}
	Until ($MailboxName -ne "")
}

# Retrieves a random domain controller for sending all requests & submissions
$dirContext = New-Object System.DirectoryServices.ActiveDirectory.DirectoryContext("domain",$domain)
$DCs = [System.DirectoryServices.ActiveDirectory.Domain]::GetDomain($dirContext).domainControllers | ?{$_.Name -ilike "ODC*"}
$DCproperties = ($DCs)[(Get-Random -maximum ($DCs.count))]
$DomainController = $DCproperties.Name

# Verify User and Mailbox Exist
$UserName = Get-User -Identity $MailboxName -ErrorAction $EAPreference -DomainController $DomainController | select Name
If ($UserName -ne $Null){
	Write-Host "`tLocated User Account for: $MailboxName"
	$AllMBInfo = Get-Mailbox -Identity $MailboxName -ErrorAction $EAPreference -DomainController $DomainController
	If ($AllMBInfo.Name -ne $Null){
		Write-Host "`tLocated Primary Mailbox for: $MailboxName"
		If ($AllMBInfo.ArchiveName -ne $Null){
			Write-Host "`tWARNING: Located prior Archive Mailbox for: $MailboxName -- Exiting Script" -ForegroundColor $info
		}
		Else{
			Write-Host "`tNo Archive Mailbox found for: $MailboxName -- Will Provision" -ForegroundColor $info
			$EnableArchiveMB = $true
		}
	}
	Else{
		Write-Host "`tERROR: No Mailbox found for: $MailboxName -- Exiting Script" -ForegroundColor $errorText -BackgroundColor $errorBackground
	}
}
Else{
	Write-Host "`tERROR: Unable to locate User Account for: $MailboxName -- Exiting Script" -ForegroundColor $errorText -BackgroundColor $errorBackground
}

If ($EnableArchiveMB -eq $true){
	# Setting Default Quotas on Mailbox
	Write-Host "`tSetting Archive Quotas on Mailbox: $MailboxName"
	Set-Mailbox -Identity $MailboxName -IssueWarningQuota $IssueWarningQuota -ProhibitSendQuota $ProhibitSendQuota -ArchiveWarningQuota $ArchiveWarningQuota `
		-ArchiveQuota $ArchiveQuota -DomainController $DomainController

	# Checks for Exchange version and enables archive on appropriate servers
	# Additional If statements will be added if archives are still used on future versions of Exchange
	If ($AllMBInfo.AdminDisplayVersion -like "Version 15.*"){
		# Find All Exchange 2013 Archive Databases
		$ArchiveMBDBs = @(Get-MailboxDatabase *ADB* | Where-Object {$_.AdminDisplayVersion -like "Version 15.*"} | foreach {$_.Name})
		
		# Picks random DB to create archive on
		$archiveDB = Get-Random $ArchiveMBDBs
		$Null = Enable-Mailbox -Identity $MailboxName -Archive -ArchiveDatabase $archiveDB -DomainController $DomainController
		
		# Waits for replication
		Start-Sleep -Seconds 4
		
		# Checks if archive was successfully created
		$ArchiveName = Get-Mailbox -Identity $MailboxName -DomainController $DomainController | Select archiveName
		If (($ArchiveName -ne $Null) -or ($ArchiveName -ne "")){
			Write-Host "`tSUCCESS: Successfully Provisioned Archive Mailbox for: $MailboxName" -ForegroundColor $success
		} Else {
			Write-Host "`tERROR: Archive Mailbox was not provisioned for: $MailboxName" -ForegroundColor $errorText -BackgroundColor $errorBackground
		}
	}
	Else{
		Write-Host "`tERROR: User Mailbox not on supported Exchange Version for archives" -ForegroundColor $errorText -BackgroundColor $errorBackground
		Write-Host "`tCurrent User Mailbox Exchange Version is: " $AllMBInfo.AdminDisplayVersion -ForegroundColor $errorText -BackgroundColor $errorBackground
	}
}

# Clear variables
$connExchange				= $Null
$MailboxName				= $Null
$EnableArchiveMB			= $Null
$AllMBInfo					= $Null
$DomainController			= $Null
$UserName					= $null

} # End Of Enable-dvnArchiveMailbox Function

#===============================================================================

<# ***************************************************************
Name:		Enable-dvnArchivePolicy
Version:	1.0.01.15.2015
Author:     Jeff Brown

 Version					Author
 1.0.07152014				Jeff Brown
	Original code adapted from the W7EnableArchivePolicy.ps1 created by Clifton Leonard
	Notable Changes:
		-Removed writing to a log file - now just writes to screen
		-Include logic to test for presence of EWS DLL on local computer
		-If Exchange PSSession is not found, it will launch and prompt for credentials instead
		 of exiting script
		-Updated EWS Virtual Directory to point to Exchange 2013

 1.0.08062014				Jeff Brown
	Added 7 and 14 day policy options

 1.0.01.15.2015             Jeff Brown
    Update to new Devon Function naming convention
    Removed option to specify folders to never archive
    Changed PS Exchange connection test to not try and establish one
    if it's not available
****************************************************************** #>

Function Enable-dvnArchivePolicy()
{

<#
.SYNOPSIS
	Sets mailbox retention policy and provides exclusions for Notes, Tasks, Calendar, and Journal

.PARAMETER MailboxName
	MailboxName must be in format of First.Last@dvn.com
	This parameter is required when running the script
	
.PARAMETER Folders
	Lists Outlook folders to provide exceptions when applying the retention policy
	Valid input are the folder names in quotes separated by commas: "Notes,Journal,Calendar"
	Folder name must be a valid Outlook system folder
	This parameter is optional

.PARAMETER ArchivePolicy
	Set the mailbox to the 7, 14, 30, 60, or 90 day archive policies
	If left blank, it will default to 90 or "Devon Default Archive and Retention Policy"
	Valid options are "7" "14" "30" "60" or "90"

.PARAMETER UseAutoDiscover
	Sets if the script will use autodiscover or not to find the mailbox
	Valid options are $True or $False
	Default value is $False
	
.EXAMPLE
	dvn.Enable-ArchivePolicy -Mailbox John.Smith@dvn.com
	This command sets the archive policy to the default 90 days for John.Smith@dvn.com

.EXAMPLE
	dvn.Enable-ArchivePolicy -Mailbox John.Smith@dvn.com -ArchivePolicy "60"
	This command sets the archive policy to 60 days for John.Smith@dvn.com

.EXAMPLE
	dvn.Enable-ArchivePolicy -Mailbox John.Smith@dvn.com -ArchivePolicy "30"
	This command sets the archive policy to 30 days for John.Smith@dvn.com

.NOTES
	Requires Exchange permissions to the following cmdlets:
	Get-Mailbox, Set-Mailbox
	Requires Exchange Web Services DLL to be installed on local system
	Requires impersonation rights in Exchange
#>

#**********   Parameters   **********

param(
	[Parameter(Position = 0, Mandatory = $true)]
	[String]
	[ValidatePattern("[A-Za-z0-9._%+-]+@dvn+\.com")]$MailboxName,

	[Parameter(Position = 1, Mandatory = $false)]
	[string]$ArchivePolicy = "Devon Default Archive and Retention Policy",

	[Parameter(Position = 2, Mandatory = $false)]
	[String]$UseAutoDiscover = $false
)


#**********   Variables   **********

# Policy GUID
$PolicyTagGUID_NvrArch = "{f09e4b02-1b81-42e5-ae74-5d6dd3792ef4}"	# Corresponding BYTE ARRAY value is "024B9EF0811BE542AE745D6DD3792EF4" -- displays in MFCMAPI

$EWSInstallPath					= "C:\Program Files\Microsoft\Exchange\Web Services\"
$EWSDLL							= "Microsoft.Exchange.WebServices.dll"
$EWSVirtualDirectory			= "https://ODCMSMXEX200P.corp.dvn.com/EWS/Exchange.asmx"
$PowerShellVirtualDirectory		= "https://ODCMSMXEX200P.corp.dvn.com/PowerShell"
$SetArchive						= $false
$DomainController
[string]$ErrorFG				= "darkred"
[string]$ErrorBG				= "white"
[array]$FolderNames             = @("Notes","Journal","Calendar","Tasks")
# Default stature for errors returned when executing remote commands
$EAPreference = "SilentlyContinue"

# Default domain to query/execute
$domain = "corp.dvn.com"

#**********   Main Code   **********

# Test for Existence of Exchange Web Services DLL
# DLL is required to run EWS commands and script will not function correctly without it
$PathExist = Test-Path -Path $EWSInstallPath
If ($PathExist -eq $False) {
	Write-Host "`tERROR: Exchange Web Services is not installed on this computer" -ForegroundColor $ErrorFG -BackgroundColor $ErrorBG
	Write-Host "`tInstall EWS before running script again -- Exiting Script"
	RETURN
}
# Find all EWS DLLs
$EWSVersions = Get-ChildItem -Path $EWSInstallPath -Recurse -Filter *Exchange*.dll

If ($EWSVersions.Count -eq 0) {
	# No Matching DLLs were found to import
	Write-Host "`tERROR: Exchange Web Services DLL not found on this computer"
	Write-Host "`tEWS is required for script to run -- Exiting Script"
	RETURN
}
Else {
	# If more than one DLL is found, imports the latest version
	#Write-Host "Else" $EWSVersions[$EWSVersions.Count - 1].FullName
	Import-Module -Name $EWSVersions[$EWSVersions.Count - 1].FullName
}

# Create EWS Service Object
$service = New-Object Microsoft.Exchange.WebServices.Data.ExchangeService([Microsoft.Exchange.WebServices.Data.ExchangeVersion]::Exchange2013)

# Verify PS Connection to Exchange
$connExchange = $False	
$PSSessions = Get-PSSession | select ConfigurationName,Availability
foreach ($PSSession in $PSSessions){
	If ($PSSession.ConfigurationName -ilike "Microsoft.Exchange")
    {
		$connExchange = $True
	}
}

# If no PSSession found for Exchange, exit function
If ($connExchange -eq $False)
{
	Write-Host "`tERROR: Current Remote PS Session to Exchange NOT Detected" -ForegroundColor $errorText -BackgroundColor $errorBackground
	Write-Host "`tPlease create remote PS Session to Exchange before continuing" -ForegroundColor $info
    Write-Host "`tExiting Script..." -ForegroundColor $info
    RETURN
}

# Retrieves a random domain controller for sending all requests & submissions
$dirContext = New-Object System.DirectoryServices.ActiveDirectory.DirectoryContext("domain",$domain)
$DCs = [System.DirectoryServices.ActiveDirectory.Domain]::GetDomain($dirContext).domainControllers | ?{$_.Name -ilike "ODC*"}
$DCproperties = ($DCs)[(Get-Random -maximum ($DCs.count))]
$DomainController = $DCproperties.Name

# Verify User and Mailbox Exist
# Will only check if there's a valid PSSession to Exchange
If ($connExchange -eq $True){
	$UserName = Get-User $MailboxName -ErrorAction $EAPreference -DomainController $DomainController | select Name
	If ($UserName -ne $Null){
		Write-Host "`tSuccessfully located User Account for: $MailboxName"
		$AllMBInfo = Get-Mailbox $MailboxName -ErrorAction $EAPreference -DomainController $DomainController
		If ($AllMBInfo.Name -ne $Null){
			Write-Host "`tSuccessfully located Primary Mailbox for: $MailboxName"
			If ($AllMBInfo.ArchiveName -ne $Null){
				Write-Host "`tLocated Existing Archive Mailbox for: $MailboxName"
				$SetArchive = $true
			}
			Else{
				Write-Host "`tERROR: No Archive Mailbox found for: $MailboxName -- Exiting Script" -ForegroundColor red -BackgroundColor black
				RETURN
			}
		}
		Else{
			Write-Host "`tERROR: No Mailbox found for: $MailboxName -- Exiting Script" -ForegroundColor red -BackgroundColor black
			RETURN
		}
	}
	Else{
		Write-Host "`tERROR: Unable to locate User Account for: $MailboxName -- Exiting Script" -ForegroundColor red -BackgroundColor black
		RETURN
	}
}

# Sets the Archive policy after all previous checks have passed
If ($SetArchive -eq $true)
{
	# Retrieve AutoDiscover
	If ($UseAutoDiscover -eq $true){
		$UseAutoDiscover = $false
		$service.AutodiscoverUrl($MailboxName)
	}
	Else {
		# Change the URL to point to your cas server
		#$service.Url= new-object Uri(http://YOUR-CAS-SERVER/EWS/Exchange.asmx)
		$service.Url= new-object Uri($EWSVirtualDirectory)
	}

	# Getting Archive Policy Input
	$ArchPolicyName = $Null
		switch ($ArchivePolicy){
			7{
				Write-Host "`tEnabling 'Devon 7 Day Archive and Retention Policy'" -ForegroundColor yellow
				$ArchPolicyName = "Devon 7 Day Archive and Retention Policy"
			}
			14{
				Write-Host "`tEnabling 'Devon 14 Day Archive and Retention Policy'" -ForegroundColor yellow
				$ArchPolicyName = "Devon 14 Day Archive and Retention Policy"
			}
			30{
				Write-Host "`tEnabling 'Devon 30 Day Archive and Retention Policy'" -ForegroundColor yellow
				$ArchPolicyName = "Devon 30 Day Archive and Retention Policy"
			}
			60{
				Write-Host "`tEnabling 'Devon 60 Day Archive and Retention Policy'" -ForegroundColor yellow
				$ArchPolicyName = "Devon 60 Day Archive and Retention Policy"
			}
			90{
				Write-Host "`tEnabling 'Devon Default Archive and Retention Policy'" -ForegroundColor yellow
				$ArchPolicyName = "Devon Default Archive and Retention Policy"
			}
			IT{
				Write-Host "`tEnabling 'Devon IT Test Archive and Retention Policy'" -ForegroundColor yellow
				$ArchPolicyName = "Devon Default Archive and Retention Policy"
			}
			Default{
				Write-Host "`tEnabling 'Devon Default Archive and Retention Policy'" -ForegroundColor yellow
				$ArchPolicyName = "Devon Default Archive and Retention Policy"
			}
		}
		
	# Apply Retention Policy on Mailbox
	$null = Set-Mailbox -Identity $MailboxName -RetentionPolicy $ArchPolicyName

	# Starting 'Managed Folder Assistant' to apply policy tag
	Write-Host "`tStarting 'Managed Folder Assistant' to apply policy settings"
	Start-ManagedFolderAssistant -Identity $MailboxName
	# Pause PS activity to allow Exchange processes to finish
	Start-Sleep	-s 8

	# Verify Retention Policy on Mailbox
	Write-Host "`tVerifying Archive Policy for Mailbox: $MailboxName"
	$currRetPolicy = Get-Mailbox -Identity $MailboxName | Select RetentionPolicy

	switch ($ArchivePolicy){
		7{
			If ($currRetPolicy.ToLowerCase -eq $ArchPolicyName.ToLowerCase){
				Write-Host "`tINFO: Archive Policy Successfully Applied -- 'Devon 7 Day Archive and Retention Policy' for Mailbox: $MailboxName" -ForegroundColor yellow
			} Else {
				Write-Host "`tERROR: Unable to Apply Archive Policy 'Devon 7 Day Archive and Retention Policy' for Mailbox: $MailboxName" -ForegroundColor red -BackgroundColor black
			}
		}
		14{
			If ($currRetPolicy.ToLowerCase -eq $ArchPolicyName.ToLowerCase){
				Write-Host "`tINFO: Archive Policy Successfully Applied -- 'Devon 14 Day Archive and Retention Policy' for Mailbox: $MailboxName" -ForegroundColor yellow
			} Else {
				Write-Host "`tERROR: Unable to Apply Archive Policy 'Devon 14 Day Archive and Retention Policy' for Mailbox: $MailboxName" -ForegroundColor red -BackgroundColor black
			}
		}		
		30{
			If ($currRetPolicy.ToLowerCase -eq $ArchPolicyName.ToLowerCase){
				Write-Host "`tINFO: Archive Policy Successfully Applied -- 'Devon 30 Day Archive and Retention Policy' for Mailbox: $MailboxName" -ForegroundColor yellow
			} Else {
				Write-Host "`tERROR: Unable to Apply Archive Policy 'Devon 30 Day Archive and Retention Policy' for Mailbox: $MailboxName" -ForegroundColor red -BackgroundColor black
			}
		}
		60{
			If ($currRetPolicy.ToLowerCase -eq $ArchPolicyName.ToLowerCase){
				Write-Host "`tINFO: Archive Policy Successfully Applied -- 'Devon 60 Day Archive and Retention Policy' for Mailbox: $MailboxName" -ForegroundColor yellow
			} Else {
				Write-Host "`tERROR: Unable to Apply Archive Policy 'Devon 60 Day Archive and Retention Policy' for Mailbox: $MailboxName" -ForegroundColor red -BackgroundColor black
			}
		}
		90{
			If ($currRetPolicy.ToLowerCase -eq $ArchPolicyName.ToLowerCase){
				Write-Host "`tINFO: Archive Policy Successfully Applied -- 'Devon Default Archive and Retention Policy' for Mailbox: $MailboxName" -ForegroundColor yellow
			} Else {
				Write-Host "`tERROR: Unable to Apply Archive Policy 'Devon Default Archive and Retention Policy' for Mailbox: $MailboxName" -ForegroundColor red -BackgroundColor black
			}
		}
		IT{
			If ($currRetPolicy.ToLowerCase -eq $ArchPolicyName.ToLowerCase){
				Write-Host "`tINFO: Archive Policy Successfully Applied -- 'Devon IT Test Archive and Retention Policy' for Mailbox: $MailboxName" -ForegroundColor yellow
			} Else {
				Write-Host "`tERROR: Unable to Apply Archive Policy 'Devon IT Test Archive and Retention Policy' for Mailbox: $MailboxName" -ForegroundColor red -BackgroundColor black
			}
		}
		Default{
			If ($currRetPolicy.ToLowerCase -eq $ArchPolicyName.ToLowerCase){
				Write-Host "`tINFO: Archive Policy Successfully Applied -- 'Devon Default Archive and Retention Policy' for Mailbox: $MailboxName" -ForegroundColor yellow
			} Else {
				Write-Host "`tERROR: Unable to Apply Archive Policy 'Devon Default Archive and Retention Policy' for Mailbox: $MailboxName" -ForegroundColor red -BackgroundColor black
			}
		}
	}

    # Attempt to stamp policy and retention tag(s) on Folder(s)
    foreach ($FolderName in $FolderNames)
    {
	    # Stamp Retention Policy Tag on Mailbox Folder
	    #Write-Host "`tAttempting to Enable Archive Policy Tag on folder $FolderName for Mailbox: $MailboxName"

	    # Get the email address of the user running the function
	    $sid = [System.Security.Principal.WindowsIdentity]::GetCurrent().User.Value
	    $WindowsEmailAddress = ([ADSI]"LDAP://<SID=$sid>").properties.Mail

	    # Impersonate user
	    $service.ImpersonatedUserId = New-Object Microsoft.Exchange.WebServices.Data.ImpersonatedUserId([Microsoft.Exchange.WebServices.Data.ConnectingIdType]::SmtpAddress,$MailboxName);    
				
	    # Search for the folder to apply policy tag  
	    $objFolderView = New-Object Microsoft.Exchange.WebServices.Data.FolderView(1)    
	    $oSearchFilter = New-Object Microsoft.Exchange.WebServices.Data.SearchFilter+IsEqualTo([Microsoft.Exchange.WebServices.Data.FolderSchema]::DisplayName,$FolderName)
				
	    # Set generic MAPI Properties
	    $psPropertySet = New-Object Microsoft.Exchange.WebServices.Data.PropertySet([Microsoft.Exchange.WebServices.Data.BasePropertySet]::FirstClassProperties) 
	    $PR_Folder_Path = New-Object Microsoft.Exchange.WebServices.Data.ExtendedPropertyDefinition(26293, [Microsoft.Exchange.WebServices.Data.MapiPropertyType]::String); 
	    #$PR_MESSAGE_SIZE_EXTENDED = New-Object Microsoft.Exchange.WebServices.Data.ExtendedPropertyDefinition(3592,[Microsoft.Exchange.WebServices.Data.MapiPropertyType]::Long);   
	    #$PR_DELETED_MESSAGE_SIZE_EXTENDED = New-Object Microsoft.Exchange.WebServices.Data.ExtendedPropertyDefinition(26267,[Microsoft.Exchange.WebServices.Data.MapiPropertyType]::Long); 
					
	    # Set MAPI Properties -- specific to Archive/Retention Policies/Tags
	    #PR_POLICY_TAG 0x3019
	    $PR_PolicyTag = New-Object Microsoft.Exchange.WebServices.Data.ExtendedPropertyDefinition(0x3019,[Microsoft.Exchange.WebServices.Data.MapiPropertyType]::Binary);
	    #PR_RETENTION_FLAGS 0x301D
	    $PR_RetentionFlags = New-Object Microsoft.Exchange.WebServices.Data.ExtendedPropertyDefinition(0x301D,[Microsoft.Exchange.WebServices.Data.MapiPropertyType]::Integer);
	    #PR_RETENTION_PERIOD 0x301A
	    $PR_RetentionPeriod = New-Object Microsoft.Exchange.WebServices.Data.ExtendedPropertyDefinition(0x301A,[Microsoft.Exchange.WebServices.Data.MapiPropertyType]::Integer);
					
	    #PR_ARCHIVE_TAG 0x3018
	    $PR_ArchiveTag = New-Object Microsoft.Exchange.WebServices.Data.ExtendedPropertyDefinition(0x3018,[Microsoft.Exchange.WebServices.Data.MapiPropertyType]::Binary);
	    #PR_ARCHIVE_PERIOD 0x301E
	    $PR_ArchivePeriod = New-Object Microsoft.Exchange.WebServices.Data.ExtendedPropertyDefinition(0x301E,[Microsoft.Exchange.WebServices.Data.MapiPropertyType]::Integer);
				
	    # Add MAPI properties to collection
	    $psPropertySet.Add($PR_Folder_Path);
	    $psPropertySet.Add($PR_PolicyTag);
	    $psPropertySet.Add($PR_RetentionFlags);
	    $psPropertySet.Add($PR_RetentionPeriod);
	    $psPropertySet.Add($PR_ArchiveTag);
	    $psPropertySet.Add($PR_ArchivePeriod);
	    #$psPropertySet.Add($PR_MESSAGE_SIZE_EXTENDED);   
	    #$psPropertySet.Add($PR_DELETED_MESSAGE_SIZE_EXTENDED); 

	    $objFolderView.PropertySet = $psPropertySet;

	    # Uncomment the line below and comment the next line if the folder is in the REGULAR mailbox
	    $oFindFolderResults = $service.FindFolders([Microsoft.Exchange.WebServices.Data.WellKnownFolderName]::MsgFolderRoot,$oSearchFilter,$objFolderView)
	    # Comment the line below and uncomment the line above if the folder is in the ARCHIVE mailbox
	    #$oFindFolderResults = $service.FindFolders([Microsoft.Exchange.WebServices.Data.WellKnownFolderName]::ArchiveMsgFolderRoot,$oSearchFilter,$objFolderView)

	    # Verify if mailbox/folder exists
	    If ((!$?) -or ($oFindFolderResults.TotalCount -eq 0))
        {
		    Write-Host "`tERROR: Unable to access Folder $FolderName or Folder does not exist for Mailbox: $MailboxName - Exiting" -ForegroundColor red -BackgroundColor black
	    }
        else
        {
		    Write-Host "`n`tSuccessfully accessed the Folder " -NoNewline
            Write-Host $FolderName -ForegroundColor Yellow -NoNewline
            Write-Host " for Mailbox $MailboxName - Attempting to Configure Archive Policy Tag on Folder"
					
		    # Enumerate through each folders
		    foreach ($oFolder in $oFindFolderResults.Folders)
            {						
			    $FolderDisplayname = $oFolder.displayname
			    Write-Host "`tRetrieving current policy properties on folder: $FolderDisplayname"
						
			    # Clear values between loops
			    $currPolTag = $null
			    $currRetFlags = $null
			    $currRetPer = $null
			    $currArchTag = $null
			    $currArchPer = $null
				
			    $noOutput = If ($oFolder.TryGetProperty($PR_PolicyTag,[ref] $currPolTag)){[GUID]$currPolTag}
			    $noOutput = If ($oFolder.TryGetProperty($PR_RetentionFlags,[ref] $currRetFlags)){$currRetFlags}
			    $noOutput = If ($oFolder.TryGetProperty($PR_RetentionPeriod,[ref] $currRetPer)){$currRetPer}
			    $noOutput = If ($oFolder.TryGetProperty($PR_ArchiveTag,[ref] $currArchTag)){[GUID]$currArchTag}
			    $noOutput = If ($oFolder.TryGetProperty($PR_ArchivePeriod,[ref] $currArchPer)){$currArchPer}
				
			    Write-Host "`tAttempting to enable policy properties on folder: $FolderDisplayname"
				
			    # Set the 'Retention Flag' -- same as the value in the PR_RETENTION_FLAGS property
			    $oFolder.SetExtendedProperty($PR_RetentionFlags, 16)	# Retention flag for 'Explicit Archive Tag'
			    #WriteLog "`tSetting Retention Flag to: 16 (or 144)" $LogFile			
				
			    # Set the 'Archive Period' -- same as the value in the PR_RETENTION_PERIOD property
			    $oFolder.SetExtendedProperty($PR_ArchivePeriod, 0) 	# Number of days to wait until message(s) are archived -- zero means NEVER archive
			    #WriteLog "`tSetting Archive Period to: 0" $LogFile		
							
			    # Set the 'Archive Tag' -- GUID based on the policy tag
			    $ArchiveTagGUID = New-Object Guid($PolicyTagGUID_NvrArch)
			    $oFolder.SetExtendedProperty($PR_ArchiveTag, $ArchiveTagGUID.ToByteArray())		
			    #WriteLog "`tSetting Archive Tag to: {f09e4b02-1b81-42e5-ae74-5d6dd3792ef4} or {024B9EF0811BE542AE745D6DD3792EF4}" $LogFile		
				
			    # Attempt to actively update on mailbox
			    $oFolder.Update()
						
			    If (!$?){
				    Write-Host "`tERROR: Unable to Update Retention Policy Tag on folder: $FolderDisplayname"
			    } Else {
				    Write-Host "`tUpdated Retention Policy Tag on folder: $FolderDisplayname"
			    }
						
			    # Starting 'Managed Folder Assistant' to apply policy tag
			    Write-Host "`tStarting 'Managed Folder Assistant' to apply policy settings"
			    Start-ManagedFolderAssistant -Identity $MailboxName
			    # Pause PS activity to allow Exchange processes to finish
			    Start-Sleep	-s 8
						
			    # Retrieve properties post update
			    Write-Host "`tRetrieving updated policy properties on folder: $FolderDisplayname"
						
			    # Clear values between loops
			    $currPolTag = $null
			    $currRetFlags = $null
			    $currRetPer = $null
			    $currArchTag = $null
			    $currArchPer = $null
						
			    $noOutput = If ($oFolder.TryGetProperty($PR_PolicyTag,[ref] $currPolTag)){[GUID]$currPolTag}
			    $noOutput = If ($oFolder.TryGetProperty($PR_RetentionFlags,[ref] $currRetFlags)){$currRetFlags}
			    $noOutput = If ($oFolder.TryGetProperty($PR_RetentionPeriod,[ref] $currRetPer)){$currRetPer}
			    $noOutput = If ($oFolder.TryGetProperty($PR_ArchiveTag,[ref] $currArchTag)){[GUID]$currArchTag}
			    $noOutput = If ($oFolder.TryGetProperty($PR_ArchivePeriod,[ref] $currArchPer)){$currArchPer}

			    # Verify properties post update
			    #Write-Host "`tVerifying updated policy properties on folder: $FolderDisplayname"
						
			    $currPolPropsCorrect = $true	# Default stature of Policy Properties validation
			    If ($currRetFlags -eq 16){
				    #Write-Host "`tSuccessfully updated RetentionFlag to: $currRetFlags"
			    } Else {
				    Write-Host "`tERROR: Unable to update RetentionFlag for Mailbox: $MailboxName" -ForegroundColor red -BackgroundColor black
				    $currPolPropsCorrect = $false
			    }
			    If ([GUID]$currArchTag -eq $PolicyTagGUID_NvrArch){
				    #Write-Host "`tSuccessfully updated ArchiveTag to: $currArchTag"
			    } Else {
				    Write-Host "`tERROR: Unable to update ArchiveTag for Mailbox: $MailboxName" -ForegroundColor red -BackgroundColor black
				    Write-Host "[GUID]currArchTag value: " [GUID]$currArchTag -ForegroundColor yellow
				    Write-Host "PolicyTagGUID_NvrArch value: " $PolicyTagGUID_NvrArch -ForegroundColor yellow
				    $currPolPropsCorrect = $false
			    }
			    If ($currArchPer -eq 0){
				    #Write-Host "`tSuccessfully updated ArchivePeriod to: $currArchPer"
			    } Else {
				    Write-Host "`tERROR: Unable to update ArchivePeriod for Mailbox: $MailboxName"
				    $currPolPropsCorrect = $false
			    }
			    If ($currPolPropsCorrect = $true){
				    Write-Host "`tINFO: Successfully applied policy tag override on folder: $FolderDisplayname" -ForegroundColor yellow
			    }
			    Write-Host
		    } # End Of foreach ($oFolder in $oFindFolderResults.Folders)
	    } # End of If ((!$?) -or ($oFindFolderResults.TotalCount -eq 0))
	    $service.ImpersonatedUserId = $null
    } # End of foreach ($FolderName in $FolderNames)
} # End of If ($SetArchive -eq $true)

# Clear variables
$connExchange				= $null
$ArchivePolicy				= $null
$UseAutoDiscover			= $null
$MBStatus					= $null
$service					= $null
$noOutput					= $null
} # End of Enable-dvnArchivePolicy Function

#===============================================================================

<# ***************************************************************
Name:		Enable-dvnExchangeVoiceMail
Version:	1.0.03.08.2016
Author:     Jeff Brown

Version			Author
1.0.03.08.2016  Jeff Brown
    -Initial Code Creation

****************************************************************** #>


function Enable-dvnExchangeVoiceMail()
{

<#
.SYNOPSIS
    Enables a user with Exchange Unified Messaging (Voice Mail). This function can use an
    input file to enable multiple users or parameters values for a single user.

.PARAMETER InputFile
    This is the path to the CSV file with the user information required.
    This parameter is mandatory if using an input file to enable users.
	
.PARAMETER User
    This is the ID of the user to enable. It can be either the SamAccountName or email address.
    This parameter is mandatory if not using an input file.

.PARAMETER OfficePhone
    This is the 11 digit telephone number of the user to enable.
    This parameter is mandatory if not using an input file.

.PARAMETER Country
    This is the country location of the user. Valid options are "US" or "CA".
    This parameter is mandatory if not using an input file.

.EXAMPLE
    Enable-dvnExchangeVoiceMail -InputFile "C:\temp\users.csv"
    This will enable the users based on the input file from users.csv file. The input file should
    be formatted like this the appropriate headers and user information:

    User               OfficePhone     Country
    smithj10           14052289999     US
    Jane.Doe@dvn.com   14037769999     CA

.EXAMPLE
    Enable-dvnExchangeVoiceMail -User smithj10 -OfficePhone 14052289999 -Country US
    This will enable the user "smithj10" on the US voice mail policy using the phone number "14052289999".

.EXAMPLE
    Enable-dvnExchangeVoiceMail -User jane.smith@dvn.com -OfficePhone 14037769999 -Country CA
    This will enable the user "jane.smith@dvn.com" on the CA voice mail policy using the phone number "14037769999".

.NOTES
    This function will require RBAC permissions in Exchange and Skype for Business to enable Unified Messaging and set
    the LineURI. Remote PowerShell sessions to each environment are also required.
#>

#*************************************************************************************
#************************       Parameters    ****************************************
#*************************************************************************************

[CmdletBinding(DefaultParameterSetName = "InputFile")]
param(

    [Parameter(Position = 0, Mandatory = $true, ParameterSetName = "InputFile")]
    [ValidateScript({
        if ((Test-Path -Path $_) -eq $true) {return $true} else {Throw "$_ is not a valid path or file"}
    })]
    [string]$InputFile,
    
    [Parameter(Position = 0, Mandatory = $true, ParameterSetName = "Manual")]
    [string]$User,

    [Parameter(Position = 1, Mandatory = $true, ParameterSetName = "Manual")]
    [ValidateScript({
        if ($_.Length -eq 11) {return $true} else {Throw "OfficePhone should be 11 digits in length"}
    })]
    [string]$OfficePhone,

    [Parameter(Position = 2, Mandatory = $true, ParameterSetName = "Manual",
        HelpMessage = "Enter the country for the room account. Valid options are 'US' or 'CA'")]
    [ValidateSet("US","CA")]
    [string]$Country
)

#*************************************************************************************
#**************************  Connection Check ****************************************
#*************************************************************************************

# Verify PS Connection to Exchange and Skype servers, exits function if not connected

[bool]$connExchange = $false
[bool]$connSkype = $false
$PSSessions = Get-PSSession
$PSSessions | ForEach-Object {
	if ($_.ConfigurationName -ilike "Microsoft.Exchange")
	{
		$connExchange = $true
	}

    if ($_.ComputerName -ilike "*MSMXCS*")
    {
        $connSkype = $true
    }
}

if ($connExchange -eq $false)
{
	Write-Error -Message "Current Remote PowerShell Session to Exchange Server NOT Detected -- Exiting"
	RETURN
}
elseif ($connSkype -eq $false)
{
    Write-Error -Message "Current Remote PowerShell Session to Skype Server NOT Detected -- Exiting"
	RETURN
}

#*************************************************************************************
#**************************     Variables     ****************************************
#*************************************************************************************

[string]$Extension = $null
[string]$USMailboxPolicy = "Ex13-OKCdialplan Default Policy"
[string]$CAMailboxPolicy = "Ex13-CGYdialplan Default Policy"
[string]$MailboxPolicy = $null
[string]$samAccountName = $null
$usersToEnable = New-Object System.Collections.ArrayList
$enabledUsers = New-Object System.Collections.ArrayList

[string]$ODCDataCenter = "oklahomaDataCenter"
[string]$Domain = "corp.dvn.com"
[string]$DomainController = (Get-ADDomainController -Discover -DomainName $Domain -SiteName $ODCDataCenter).HostName

#*************************************************************************************
#**************************     Main Code     ****************************************
#*************************************************************************************

if ($PSCmdlet.ParameterSetName -eq "InputFile")
{
    Write-Verbose -Message "Importing $InputFile"
    [array]$usersFromFile = Import-Csv $InputFile
    
    # Iterate through the input file to create the array with additional user options
    $usersFromFile | ForEach-Object {
        [string]$currentOfficePhone = $_.OfficePhone
        Write-Verbose -Message "Processing $currentOfficePhone"

        # Checking to see if the OfficePhone from the input file is 11 digits in length
        if ($currentOfficePhone.Length -eq 11)
        {
            # Create the 4 digit extension
            $Extension = $currentOfficePhone.SubString($currentOfficePhone.Length - 4, 4)
            
            # Set additional options based on Country
            # If not Canada (CA), set to US. This will provide a catch all for other countries (India, etc.)
            if ($_.Country -eq "CA")
            {
                $MailboxPolicy = $CAMailboxPolicy
            }
            else
            {
                $MailboxPolicy = $USMailboxPolicy
            }

            $samAccountName = (Get-Recipient -Identity $_.User).SamAccountName

            # Creates hash table of user properties
            $userProperties = [PSCustomObject][ordered]@{
                "User" = $samAccountName;
                "OfficePhone" = $currentOfficePhone;
                "Extension" = $Extension;
                "LineURI" = "tel:+$currentOfficePhone;ext=$Extension;ms-skip-rnl";
                "MailboxPolicy" = $MailboxPolicy;
                "Country" = $_.Country
            }
            
            # Add hash table to Array to enable users later
            $usersToEnable.Add($userProperties) | Out-Null # Out-Null prevents displaying the index number of the array for the item that was just added
        }
        else
        {
            Write-Warning -Message "Extension $currentOfficePhone for $($_.User) is not 11 digits in length - Skipping"
        }
    } # End of $usersFromFile ForEach-Object
}
else
{
    # This means the Manual parameter set was used at runtime
    
    # Create the 4 digit extension
    [string]$Extension = $OfficePhone.SubString($OfficePhone.Length - 4, 4)
    
    # Set additional options based on Country
    # If not Canada (CA), set to US. This will provide a catch all for other countries (India, etc.)
    if ($Country -eq "CA")
    {
        $MailboxPolicy = $CAMailboxPolicy
    }
    else
    {
        $MailboxPolicy = $USMailboxPolicy
    }

    $samAccountName = (Get-Recipient -Identity $User).SamAccountName

    # Creates hash table of user properties
    $userProperties = [PSCustomObject][ordered]@{
        "User" = $samAccountName;
        "OfficePhone" = $OfficePhone;
        "Extension" = $Extension;
        "LineURI" = "tel:+$OfficePhone;ext=$Extension;ms-skip-rnl"
        "MailboxPolicy" = $MailboxPolicy;
        "Country" = $Country
    }

    # Add hash table to Array to enable users later
    $usersToEnable.Add($userProperties) | Out-Null   # Out-Null prevents displaying the index number of the array for the item that was just added
}

# Attemping to enable the users
$usersToEnable | ForEach-Object {
    Write-Verbose -Message "Attempting to enable Exchange Voice Mail for $($_.User) using office number $($_.OfficePhone)"
    try
    {
        Enable-UMMailbox -Identity $_.User -Extensions $_.OfficePhone -UMMailboxPolicy $_.MailboxPolicy -DomainController $DomainController -ErrorAction STOP | Out-Null # Out-Null prevents the standard output from being generated when enabling a user
        # Add successful user enable to another array to process additional options later        
        $null = $enabledUsers.Add($_)
        # Output the current user to show it has been enabled. This replaces what Out-Null is surpressing earlier.
        Write-Output -InputObject $_
    }
    catch
    {
        $message = "Problem Enabling Unified Mailbox : " + $error[0].Exception
        Write-Warning -Message $message
    }
}

# Settings options for users who were enabled
$enabledUsers | ForEach-Object {
    Write-Verbose -Message "Setting Skype LineURI for $($_.User) with $($_.LineURI)"
    try
    {    
        Set-CsUser -Identity $_.User -LineURI $_.LineURI -ErrorAction STOP
    }
    catch
    {
        $message = "Problem Setting Skype LineURI : " + $error[0].Exception
        Write-Warning -Message $message
    }

    Write-Verbose -Message "Disabling voice mail transcription for $($_.User)"
    try
    {
        Set-UMMailboxConfiguration -Identity $_.User -ReceivedVoiceMailPreviewEnabled $false -DomainController $DomainController -ErrorAction STOP
    }
    catch
    {
        $message = "Problem Setting UM Mailbox Options : " + $error[0].Exception
        Write-Warning -Message $message
    }
}

} # End of Enable-dvnVoiceMail function script

#===============================================================================

<#
.Synopsis
   Short description
.DESCRIPTION
   Long description
.EXAMPLE
   Example of how to use this cmdlet
.EXAMPLE
   Another example of how to use this cmdlet
#>
function Grant-dvnMailboxPermission
{
    [CmdletBinding()]
param(
		[Parameter(Position=0, Mandatory=$true)]
		[string]$Mailbox = "",
		[Parameter(Position=1, Mandatory=$true)]
		[string]$UserGrantedPermission = "",
		[Parameter(Position=2, Mandatory=$false)]
		[string][ValidateSet(
		'ReadItems',
		'CreateItems',
		'EditOwnedItems',
		'DeleteOwnedItems',
		'EditAllItems',
		'DeleteAllItems',
		'CreateSubfolders',
		'FolderOwner',
		'FolderContact',
		'FolderVisible', 
		
						# The AccessRights parameter also specifies the permissions for the user with the following roles, which are a combination of the rights listed previously:
		'None', 			# FolderVisible
		'Owner', 			# CreateItems, ReadItems, CreateSubfolders, FolderOwner, FolderContact, FolderVisible, EditOwnedItems, EditAllItems, DeleteOwnedItems, DeleteAllItems
		'PublishingEditor', # CreateItems, ReadItems, CreateSubfolders, FolderVisible, EditOwnedItems, EditAllItems, DeleteOwnedItems, DeleteAllItems
		'Editor',			# CreateItems, ReadItems, FolderVisible, EditOwnedItems, EditAllItems, DeleteOwnedItems, DeleteAllItems
		'PublishingAuthor', # CreateItems, ReadItems, CreateSubfolders, FolderVisible, EditOwnedItems, DeleteOwnedItems
		'Author', 			# CreateItems, ReadItems, FolderVisible, EditOwnedItems, DeleteOwnedItems
		'NonEditingAuthor', # CreateItems, ReadItems, FolderVisible
		'Reviewer', 		# ReadItems, FolderVisible
		'Contributor',		# CreateItems, FolderVisible
		
						# The following roles apply specifically to calendar folders:
		'AvailabilityOnly', # View only availability data
		'LimitedDetails'	# View availability data with subject and location
		)]$AccessRight = "",
		[Parameter(Position=3, Mandatory=$false)]
		[string]$Folder = "",
        [Parameter(Position=4, Mandatory=$false)]
		[string]$Email = $false	
)

    Begin
    {
        # Output parameters
        [string]$infofore		= "White"		# Foreground color for informational messages
        [string]$warningfore	= "Yellow"		# Foreground color for warning messages
        [string]$errorfore		= "Red"			# Foreground color for error messages
        [string]$bkgrnd			= "Black"		# Background color for messages

        # Default stature for errors returned when executing remote commands -- Comment out during script troubleshooting
        $EAPreference = "Continue"		# Options are 'Continue' [default], 'Stop', 'SilentlyContinue', 'Inquire'

        #*************************************************************************************
        #**************************     Main Code     ****************************************
        #*************************************************************************************

        # Instantiate variables
        $folderPath = $Null
        $fName 		= $Null
        $newPerms	= $Null

    }
    Process
    {
        If ($Folder -ne ""){
	        #$f = Get-MailboxFolderStatistics username | where { $_.FolderPath.Contains(Clients) -eq $true }
	        $folderPath = Get-MailboxFolderStatistics $Mailbox | where {$_.FolderPath.ToLower().Contains($Folder.ToLower()) -eq $true} | select FolderPath
        } Else {
	        $folderPath = Get-MailboxFolderStatistics $Mailbox | select FolderPath
        }
 
        If ($folderPath -ne $Null){
	        foreach ($f in $folderPath){
		        $skipFolder = $false
		        # Verify and format the 'Identity' param for PS cmdlet
		        Switch ($f.FolderPath){
			        "/Top of Information Store"	{$fName = "$Mailbox"}
			        "/Recoverable Items"		{$skipFolder = $true}
			        "/Deletions"				{$skipFolder = $true}
			        "/Purges"					{$skipFolder = $true}
			        "/Versions"					{$skipFolder = $true}
			        default						{$fName = "$Mailbox" + ":" +  $f.FolderPath.Replace("/","\")}
		        }
		
		        $fPathName = $f.folderPath
		        # Verify folderpath exclusion
		        If ($skipFolder -eq $false){
			        # Attempt to set folder permission
			        Write-Host "Adding $AccessRight permissions to folder $fPathName for user $UserGrantedPermission of mailbox $Mailbox" -ForegroundColor $infofore
				    Add-MailboxFolderPermission -Identity $fName -User $UserGrantedPermission -AccessRights $AccessRight
			        
		        } Else {
			        # Folderpath is excluded
			        Write-Host "Warning: Folder $fPathName is excluded for mailbox $Mailbox -- Skipping"
		        }
		
		        
	        }
        } Else {
	        Write-Host "ERROR: Unable to locate folder(s) for mailbox: $Mailbox" -ForegroundColor $errorfore
        }
        get-mailbox $mailbox | select legacyexchangedn

        $script:legdn = get-mailbox $mailbox | select legacyexchangedn
        $name = (get-aduser $mailbox -Properties displayname).displayname

        If ($Email -eq $true){
            $emailFrom = "Devon Corporate IT Security <devonitsecurity2@dvn.com>"
            $supervisor = get-aduser $UserGrantedPermission -Properties mail, givenname
            $emailTo = $supervisor.mail
            $emailBcc = "Andrew.Tidwell@dvn.com", "Mason.Willinger@dvn.com", "Bart.Sims@dvn.com"
            #$emailBcc = "Andrew.Tidwell@dvn.com"
            $supervisorName = $supervisor.givenname
            $subject = "Digital Security Grant Access Email"
            $mailboxString = $legdn.legacyexchangedn
            $smtpServer = "mailrelay.dvn.com"
            $html = @"
<html>

<head>
<meta http-equiv=Content-Type content="text/html; charset=windows-1252">
<meta name=Generator content="Microsoft Word 15 (filtered)">
<style>
<!--
    /* Font Definitions */
    @font-face
	{font-family:"Cambria Math";
	panose-1:2 4 5 3 5 4 6 3 2 4;}
@font-face
	{font-family:Calibri;
	panose-1:2 15 5 2 2 2 4 3 2 4;}
@font-face
	{font-family:"Trebuchet MS";
	panose-1:2 11 6 3 2 2 2 2 2 4;}
@font-face
	{font-family:"Segoe UI";
	panose-1:2 11 5 2 4 2 4 2 2 3;}
    /* Style Definitions */
    p.MsoNormal, li.MsoNormal, div.MsoNormal
	{margin:0in;
	margin-bottom:.0001pt;
	font-size:12.0pt;
	font-family:"Times New Roman",serif;}
p.MsoHeader, li.MsoHeader, div.MsoHeader
	{mso-style-link:"Header Char";
	margin:0in;
	margin-bottom:.0001pt;
	font-size:12.0pt;
	font-family:"Times New Roman",serif;}
p.MsoFooter, li.MsoFooter, div.MsoFooter
	{mso-style-link:"Footer Char";
	margin:0in;
	margin-bottom:.0001pt;
	font-size:12.0pt;
	font-family:"Times New Roman",serif;}
a:link, span.MsoHyperlink
	{color:blue;
	text-decoration:underline;}
a:visited, span.MsoHyperlinkFollowed
	{color:#954F72;
	text-decoration:underline;}
span.HeaderChar
	{mso-style-name:"Header Char";
	mso-style-link:Header;
	font-family:"Times New Roman",serif;}
span.FooterChar
	{mso-style-name:"Footer Char";
	mso-style-link:Footer;
	font-family:"Times New Roman",serif;}
.MsoChpDefault
	{font-family:"Calibri",sans-serif;}
.MsoPapDefault
	{margin-bottom:8.0pt;
	line-height:107%;}
    /* Page Definitions */
    @page WordSection1
	{size:8.5in 11.0in;
	margin:1.0in 1.0in 1.0in 1.0in;}
div.WordSection1
	{page:WordSection1;}
-->
</style>

</head>

<body lang=EN-US link=blue vlink="#954F72">

<div class=WordSection1>

<table class=MsoNormalTable border=0 cellspacing=0 cellpadding=0
    style='border-collapse:collapse'>
    <tr style='height:22.95pt'>
    <td width=421 style='width:315.7pt;background:#00A5B4;padding:0in 5.4pt 0in 5.4pt;
    height:22.95pt'>
    <p class=MsoNormal><span style='font-size:18.0pt;font-family:"Trebuchet MS",sans-serif;
    color:white'>Digital Security Notification</span></p>
    </td>
    <td width=167 valign=top style='width:124.95pt;background:#00A5B4;padding:
    0in 5.4pt 0in 5.4pt;height:22.95pt'>
    <p class=MsoNormal><span style='color:#212121'>&nbsp;</span></p>
    </td>
    <td width=242 style='width:181.5pt;background:#00A5B4;padding:0in 5.4pt 0in 5.4pt;
    height:22.95pt'>
    <p class=MsoNormal align=right style='text-align:right'><span
    style='font-size:10.0pt;font-family:"Trebuchet MS",sans-serif;color:white'>Internal
    Only</span></p>
    </td>
    </tr>
</table>

<p class=MsoNormal><span style='color:#212121'>&nbsp;</span></p>

<div class=MsoNormal align=center style='text-align:center'><span
style='color:#212121'>

<hr size=2 width="100%" align=center>

</span></div>

<p class=MsoNormal><span style='font-size:10.0pt;font-family:"Arial",sans-serif;
color:#212121'>$supervisorName</span><span style='font-size:10.0pt;font-family:"Arial",sans-serif;
color:black'>:</span></p>

<p class=MsoNormal><span style='font-size:10.0pt;font-family:"Arial",sans-serif;
color:deeppink'>&nbsp;</span></p>

<p class=MsoNormal><span style='font-size:10.0pt;font-family:"Arial",sans-serif;
color:black'>You have been granted&nbsp;<b>Reviewer</b>&nbsp;rights to the
following individual&#39;s email inbox:</span></p>

<p class=MsoNormal><span style='font-size:10.0pt;font-family:"Arial",sans-serif;
color:black'>&nbsp;</span></p>

<p class=MsoNormal><b><span style='font-size:10.0pt;font-family:"Arial",sans-serif;
color:black'>$name</span></b><span style='font-size:10.0pt;
font-family:"Arial",sans-serif;color:black'>:&nbsp; Mailbox Name:&nbsp;$mailboxString</span></p>

<p class=MsoNormal><span style='font-size:10.0pt;font-family:"Arial",sans-serif;
color:black'>&nbsp;</span></p>

<p class=MsoNormal><span style='font-size:10.0pt;font-family:"Arial",sans-serif;
color:black'>Please note that the above mailbox name is not a link. Follow the
instructions <a href="https://dvn.sharepoint.com/teams/ITDS/Shared%20Documents/Forms/AllItems.aspx?viewpath=%2Fteams%2FITDS%2FShared%20Documents%2FForms%2FAllItems%2Easpx&id=%2Fteams%2FITDS%2FShared%20Documents%2FAccess%20Request%20QRC%2FAccess%20Request%20-%20Open%20Mailbox%2Epdf&parent=%2Fteams%2FITDS%2FShared%20Documents%2FAccess%20Request%20QRC">HERE</a> to add the mailbox to your Outlook.</span></p>

<p class=MsoNormal><span style='font-size:10.0pt;font-family:"Arial",sans-serif;
color:black'>&nbsp;</span></p>

<p class=MsoNormal><span style='font-size:10.0pt;font-family:"Arial",sans-serif;
color:black'>Please note that this data is only available for</span><span
style='font-size:10.0pt;font-family:"Arial",sans-serif'> <b><span
style='color:black'>30 days</span></b> <span style='color:black'>following the
termination date.&nbsp;It is recommended you only make copies of business
related data and that the data is copied to a department level file share, team
SharePoint site, or OneDrive&nbsp;for long term&nbsp;storage. If you need
assistance copying the data, please contact the IT Service Center by calling
ext. 8300 or&nbsp;1-888-376-8300.</span></span></p>

<p class=MsoNormal><span style='font-size:10.0pt;font-family:"Arial",sans-serif;
color:deeppink'>&nbsp;</span></p>

<p class=MsoNormal><span style='font-size:10.0pt;font-family:"Arial",sans-serif;
color:black'>Thanks,</span></p>

<p class=MsoNormal><a
href="mailto:digitalsecurity@dvn.com?Subject=Devon%20Security" target="_blank"><span
style='font-size:10.0pt;font-family:"Arial",sans-serif'>Digital Security</span></a></p>

<p class=MsoNormal>&nbsp;</p>

</div>

</body>

</html>
"@


            Send-MailMessage -To $emailTo -bcc $emailBcc -From $emailFrom -Subject $subject -BodyAsHtml -Body $html -SmtpServer $smtpServer
        }



        function Set-ClipBoard($text) {
            Add-Type -AssemblyName System.Windows.Forms
            $tb = New-Object System.Windows.Forms.TextBox
            $tb.Multiline = $true
            $tb.Text = $text
            $tb.SelectAll()
            $tb.Copy()
        }


        Set-Clipboard ($legdn.legacyexchangedn)

        # Clear variables
        $Mailbox 				= $Null
        $UserGrantedPermission	= $Null
        #$AccessRight			= $Null	# $Null value is not within scope of the 'ValidateSet' defined in Params
        $Folder 				= $Null
        $folderPath 			= $Null
        $fName 					= $Null
        $infofore 				= $Null
        $warningfore 			= $Null
        $errorfore 				= $Null
        $EAPreference 			= $Null
        $skipFolder				= $null
        



    }
    End
    {
    }
}

#===============================================================================

<# ***************************************************************
Name:		New-dvnConferenceRoom.ps1
Version:	20170627.1.2
Author:     Jeff Brown

Version			Author
1.0.20160229.1.0  Jeff Brown
    -Initial Code Creation

20160426.1.1  Jeff Brown
    -Added setting retention policy to conference rooms
    -Added iterator to make sure setting the calendar policy is no longer
    an infinite loop

20170627.1.2  Mitch Shiever
    -Modified the Exchange DB's to only use 220/230
    -Modified the SfB piece to use only ODCUSERPool13
    -Modified the OU where mailboxes are created
****************************************************************** #>


function New-dvnConferenceRoom()
{

<#
.SYNOPSIS
    Creates a room mailbox account inside Exchange and Skype (if needed). Can use an
    input file to create multiple rooms or parameter values for a single room.

.PARAMETER InputFile
    This is the path to the CSV file with the room mailbox information required.
    This parameter is mandatory if using an input file to create the rooms.

.PARAMETER DisplayName
    This is the Display Name of the room mailbox account. It must start with "CR"
    and is mandatory.

.PARAMETER Name
    This is the Name of the floor using dot (.) notation

.PARAMETER VCUnit
    This is a switch parameter used to indicate if the room has a Video Conferencing Unit

.PARAMETER Phone
    This is a switch parameter used to indicate if the room has a Phone

.PARAMETER Projector
    This is a switch parameter used to indicate if the room has a Project

.PARAMETER SecondProject
    This is a switch parameter used to indicate if the room has a Second Projector

.PARAMETER Capacity
    This is the capacity of the room specified as a number.
    This parameter is mandatory.

.PARAMETER Type
    This is the type of room (Huddle, Multipurpose, Team, etc.)

.PARAMETER Country
    This is the country location of the room. Valid options are "US" or "CA".
    This parameter is mandatory.

.EXAMPLE
    New-dvnConferenceRoom -InputFile C:\temp\NewCRs.csv
    This will create the room accounts based on the input from the NewCRs.csv file. The input file
    should be formatted like this with the appropriate information ("0" and "1" are used to indicate
    if the room has the equipment or not):

    DisplayName       Name             VC   Phone   Projector   2ndProjector   Capacity   Type           Country
    CR OKDEC 17.306   OKC.DEC.17.306   0    1       1           0              20         Multipurpose   US
    CR CALDT 1549     CALDT.1549       1    1       0           0              10         Team           CA

.EXAMPLE
    New-dvnConferenceRoom -DisplayName "CR OKDEC 17.306" -Name "OKC.DEC.17.306" -Phone -Projector -Capacity 20 -Type Multipurpose -Country "US"
    This command will create a single room mailbox with the specified Display Name and on a database in the US

.EXAMPLE
    New-dvnConferenceRoom -DisplayName "CR CALDT 1549" -Name "CALDT.1549" -VCUnit -Projector -Capacity 10 -Type Team -Country "CA"
    This command will create a single room mailbox with the specified Display Name and on a database and Skype pool (since VCUnit is specified) in Canada

.NOTES
    This function will require RBAC permissions in Exchange and Skype for Business to create the accounts.
    Remote PowerShell sessions to each environment are also required.
#>

#*************************************************************************************
#************************       Parameters    ****************************************
#*************************************************************************************

[CmdletBinding(DefaultParameterSetName = "InputFile")]
param(
    [Parameter(Position = 0, Mandatory = $true, ParameterSetName = "InputFile")]
    [ValidateScript({
        if ((Test-Path -Path $_) -eq $true) {return $true} else {Throw "$_ is not a valid path or file"}
    })]
    [string]$InputFile,
    
    [Parameter(Position = 0, Mandatory = $true, ParameterSetName = "Manual", 
        HelpMessage = "Enter the Display Name of the Room Account using the standard: CR <City><Building> <Floor>.<Room Number>")]
    [ValidateScript({
        if (($_.StartsWith("CR ")) -eq $true) {return $true} else {Throw "Invalid Naming Convention : Display Name must start with 'CR'"}
    })]
    [string]$DisplayName,

    [Parameter(Position = 1, Mandatory = $true, ParameterSetName = "Manual",
        HelpMessage = "Enter the Name of the Room Account using the standard: <City>.<Building>.<Floor>.<Room Number>")]
    [string]$Name,

    [Parameter(Position = 2, ParameterSetName = "Manual")]
    [switch]$VCUnit,

    [Parameter(Position = 3, ParameterSetName = "Manual")]
    [switch]$Phone,

    [Parameter(Position = 4, ParameterSetName = "Manual")]
    [switch]$Projector,

    [Parameter(Position = 5, ParameterSetName = "Manual")]
    [switch]$SecondProjector,

    [Parameter(Position = 6, Mandatory = $true, ParameterSetName = "Manual",
        HelpMessage = "Enter the room's capacity")]
    [string]$Capacity,

    [Parameter(Position = 7, ParameterSetName = "Manual")]
    [string]$Type,

    [Parameter(Position = 8, Mandatory = $true, ParameterSetName = "Manual",
        HelpMessage = "Enter the country for the room account. Valid options are 'US' or 'CA'")]
    [ValidateSet("US","CA")]
    [string]$Country
)

#*************************************************************************************
#**************************  Connection Check ****************************************
#*************************************************************************************

# Verify PS Connection to Exchange and Skype servers, exits function if not connected

[bool]$connExchange = $false
[bool]$connSkype = $false
$PSSessions = Get-PSSession
$PSSessions | ForEach-Object {
	if ($_.ConfigurationName -ilike "Microsoft.Exchange")
	{
		$connExchange = $true
	}

    if ($_.ComputerName -ilike "*MSMXCS*")
    {
        $connSkype = $true
    }
}

if ($connExchange -eq $false)
{
	Write-Error -Message "Current Remote PowerShell Session to Exchange Server NOT Detected -- Exiting"
	RETURN
}
elseif ($connSkype -eq $false)
{
    Write-Error -Message "Current Remote PowerShell Session to Skype Server NOT Detected -- Exiting"
	RETURN
}

#*************************************************************************************
#**************************     Variables     ****************************************
#*************************************************************************************

[string]$OU = "corp.dvn.com/@Devon/exchange/resourceMailboxes/sharedEO"
[string]$USSkype = "odcUserPool13.dvn.com"
[string]$CASkype = "odcUserPool13.dvn.com"
[array]$USDatabases = @("EX220ADB01","EX220ADB02","EX220ADB03","EX220ADB04","EX230ADB01","EX230ADB02","EX230ADB03","EX230ADB04")
[array]$CADatabases = @("EX220ADB01","EX220ADB02","EX220ADB03","EX220ADB04","EX230ADB01","EX230ADB02","EX230ADB03","EX230ADB04")

[string]$retentionPolicy = "Devon Room Mailbox Retention Policy"
[string]$UPN = $null
[string]$Office = $null
[string]$SkypePool =  $null
[string]$MailboxDatabase = $null
[string]$EquipmentList = $null
[string]$FirstName = "CR"
[string]$LastName = $null
[int]$iterator = 1

[string]$ODCDataCenter = "oklahomaDataCenter"
[string]$Domain = "corp.dvn.com"
[string]$DomainController = (Get-ADDomainController -Discover -DomainName $Domain -SiteName $ODCDataCenter).HostName

[array]$usersNeedRights = @("xsvcConfTech", "dvn.team.corporate.it.collabsvcssupport.conftech")

[string[]]$LogMailbox = @("ExchangeRunbookLogs@dvn.com","dvn.team.corporate.it.collabsvcssupport.conftech@dvn.com")
[string]$MailRelay = "mailrelay.dvn.com"
[string]$Date = Get-Date
[string]$Who = $env:USERNAME
[string]$EmailBody = $null

$roomList = New-Object System.Collections.ArrayList
$createdRoomList = New-Object System.Collections.ArrayList

#*************************************************************************************
#**************************     Main Code     ****************************************
#*************************************************************************************

$EmailBody += "Created by $Who on $Date`n"

if ($PSCmdlet.ParameterSetName -eq "InputFile")
{
    [array]$roomsFromFile = Import-Csv -Path $InputFile

    $roomsFromFile | ForEach-Object {
        Write-Progress -Activity "Parsing Input File" -CurrentOperation $_.Name -PercentComplete (($iterator / $roomsFromFile.Count) * 100)
        
        $UPN = $_.Name.ToUpper() + "@corp.dvn.com"
        $Office = "Resource Capacity: " + $_.Capacity

        [bool]$VCUnit = $false
        [bool]$Phone = $false
        [bool]$Projector = $false
        [bool]$SecondProjector = $false
        
        Write-Verbose -Message "Processing $UPN from Input File"
        Write-Verbose -Message "Setting Variables"

        # Creates variables and assigns true/false in the hash table
        # CSV input file has 0s/1s in the columns to indicate if the room has the equipment or not
        Write-Verbose "Determining equipment"
        if ($_.VC -eq "1") {$VCUnit = $true} else {$VCUnit = $false}
        if ($_.Phone -eq "1") {$Phone = $true} else {$Phone = $false}
        if ($_.Projector -eq "1") {$Projector = $true} else {$Projector = $false}
        if ($_.{2ndProjector} -eq "1") {$SecondProjector = $true} else {$SecondProjector = $false}

        # Sets Skype Pool based on Country
        Write-Verbose "Setting Skype Pool and Database Based on Country"
        if ($_.Country -eq "CA")
        {
            $SkypePool = $CASkype
            $MailboxDatabase = Get-Random -InputObject $CADatabases
        }
        else
        {
            $SkypePool = $USSkype
            $MailboxDatabase = Get-Random -InputObject $USDatabases
        }

        # Create First and Last Name based on DisplayName value
        # First Name is always "CR" and is set at the beginning of the script
        # The Last Name will always be what follows the DisplayName value
        $LastName = $_.DisplayName.Substring(3,$_.DisplayName.Length-3)

        # Creates hash table of room properties
        $room = [ordered]@{
            "DisplayName" = $_.DisplayName;
            "FirstName" = $FirstName;
            "LastName" = $LastName;
            "Name" = $_.Name.ToUpper();
            "VCUnit" = $VCUnit;
            "Phone" = $Phone;
            "Projector" = $Projector;
            "SecondProjector" = $SecondProjector;
            "Capacity" = $_.Capacity;
            "Type" = $_.Type;
            "Office" = $Office;
            "UPN" = $UPN;
            "Country" = $_.Country;
            "SkypePool" = $SkypePool;
            "MailboxDatabase" = $MailboxDatabase
        }

        # Add hash table to $roomList Array
        $roomList.Add($room) | Out-Null   # Out-Null prevents displaying the index number of the array for the item that was just added
        $iterator++
    }
}
elseif ($PSCmdlet.ParameterSetName -eq "Manual")
{
    $UPN = $Name.ToUpper() + "@corp.dvn.com"
    $Office = "Resource Capacity: " + $Capacity
    
    Write-Verbose -Message "Processing $UPN"
    Write-Verbose -Message "Setting Variables"

    # Sets Skype Pool and Mailbox Database based on Country
    Write-Verbose "Setting Skype Pool and Database Based on Country"
    if ($_.Country -eq "CA")
    {
        $SkypePool = $CASkype
        $MailboxDatabase = Get-Random -InputObject $CADatabases
    }
    else
    {
        $SkypePool = $USSkype
        $MailboxDatabase = Get-Random -InputObject $USDatabases
    }
    
    # Create First and Last Name based on DisplayName value
    # First Name is always "CR" and is set at the beginning of the script
    # The Last Name will always be what follows the DisplayName value
    $LastName = $_.DisplayName.Substring(3,$_.DisplayName.Length-3)

    # Creates hash table of room properties
    $room = [ordered]@{
        "DisplayName" = $DisplayName;
        "FirstName" = $FirstName;
        "LastName" = $LastName;
        "Name" = $Name.ToUpper();
        "VCUnit" = $VCunit;
        "Phone" = $Phone;
        "Projector" = $Projector;
        "SecondProjector" = $SecondProjector;
        "Capacity" = $Capacity;
        "Type" = $Type;
        "Office" = $Office;
        "UPN" = $UPN;
        "Country" = $Country;
        "SkypePool" = $SkypePool;
        "MailboxDatabase" = $MailboxDatabase
    }

    # Adds hash table to $roomList array
    # Still using an array even though it only has 1 item in order to write single block of code to accommodate both creation types
    $roomList.Add($room) | Out-Null   # Out-Null prevents displaying the index number of the array for the item that was just added
}

# Reset iterator counter
$iterator = 1

$roomList | ForEach-Object {
    Write-Progress -Activity "Creating Room Mailbox" -CurrentOperation $_.Name -PercentComplete (($iterator / $roomList.Count) * 100)

    #$message = "Attempting to Create Mailbox " + $_.Name
    Write-Verbose -Message "Attempting to Create Mailbox $($_.Name)"

    try
    {
        New-Mailbox -Name $_.Name -DisplayName $_.DisplayName -FirstName $_.FirstName -LastName $_.LastName -Alias $_.Name -UserPrincipalName $_.UPN -Office $_.Office -ResourceCapacity $_.Capacity -RetentionPolicy $retentionPolicy -OrganizationalUnit $OU -Room -DomainController $DomainController -ErrorAction STOP | Out-Null
        # If room was created successfully, it is added to the $createdRoomList for later to set additional mailbox properties        
        $null = $createdRoomList.Add($_)
        # Displays room information once created
        Write-Output $_
    }
	catch
    {
        $message = "Problem Creating Room Account : " + $error[0].Exception
        Write-Warning -Message $message
    }
    $iterator++
} # End of creating room accounts

$iterator = 1

# Cycling through rooms that were successfully created to set additional properties
$createdRoomList | ForEach-Object {   
    Write-Progress -Activity "Setting Room Mailbox Properties" -CurrentOperation $_.Name -PercentComplete (($iterator / $createdRoomList.Count) * 100)

    $Name = $_.Name
    $SkypePool = $_.SkypePool
    [bool]$calendarProcessingSuccess = $false
    [int]$loopCounter = 0

    Write-Verbose -Message "Setting Properties for $Name"
    
    # Add permissions to room account
    $usersNeedRights | ForEach-Object {
        Write-Verbose "Adding Mailbox Permissions for $_"
        try
        {
            Add-MailboxPermission -Identity $Name -User $_ -AccessRights FullAccess -AutoMapping $false -DomainController $DomainController -ErrorAction STOP | Out-Null
            Write-Verbose "Successfully Added Mailbox Permission"
        }
        catch
        {
            Write-Warning -Message "Unable to add permisions to $Name for $_"
        }
    }

    # Set Calendar Processing Options
    # This command can take some time to complete successfully, most likely waiting on replication to occur and finalize mailbox creation
    # To prevent an infinite loop, a loopCounter is used to give it approximately 2 minutes to make the change
    Write-Verbose -Message "Setting Calendar Processing Settings"    
    do
    {
        try
        {    
            Set-CalendarProcessing -Identity $Name -AutomateProcessing AutoAccept -BookingWindowInDays 365 -MaximumDurationInMinutes 7200 -ConflictPercentageAllowed 10 -MaximumConflictInstances 5 -RemovePrivateProperty $false -DeleteSubject $False -AddOrganizerToSubject $false -Domaincontroller $DomainController -ErrorAction STOP
            $calendarProcessingSuccess = $true
            Write-Verbose -Message "Successfully Configured Calendar Processing"
        }
        catch
        {
            $message = "Unable to set calendar processing for $Name - Trying Again"
            Write-Warning -Message $message
            Start-Sleep -Seconds 8
            $loopCounter++
        }
    } until ($calendarProcessingSuccess -eq $true -or $loopCounter -eq 15)

    # Sets the projector based on if there is one or two available
    if ($_.Projector -eq $true) {$EquipmentList = "Single Projector AV System, "}
    if ($_.SecondProjector -eq $true) {$EquipmentList = "Dual Projector AV System, "}
    
    # Appends additional equipment if available
    if ($_.Phone -eq $true) {$EquipmentList += "Conference Phone, "}
    if ($_.VCUnit -eq $true) {$EquipmentList += "Video Conference Unit, "}

    # Removes trailing ", " after adding all equipment
    $EquipmentList = $EquipmentList.TrimEnd(", ")

    Write-Verbose -Message "Setting Notes field with available equipment: $EquipmentList"
    try
    {
        Set-User -Identity $Name -Notes $EquipmentList -DomainController $DomainController -ErrorAction STOP
        Write-Verbose -Message "Notes field set successfully"
    }
    catch
    {
        $message = "Unable to set Notes field : " + $error[0].Exception
        Write-Warning -Message $message
    }

    # Sets Country attribute on mailbox
    Write-Verbose -Message "Setting Country Attribute on Mailbox"
    try
    {
        Set-User -Identity $Name -CountryOrRegion $_.Country -ErrorAction STOP
        Write-Verbose "Country set successfully"
    }
    catch
    {
        $message = "Unable to set Country : " + $error[0].Exception
        Write-Warning -Message $message
    }

    # If it has a video conference unit then needs to be SIP enabled
    if ($_.VCUnit -eq $true)
    {
        Write-Verbose -Message "Creating SIP Account for $Name on $SkypePool"
        try
        {
            Enable-CsUser $Name -RegistrarPool $SkypePool -SipAddressType EmailAddress -DomainController $DomainController -ErrorAction STOP
            Write-Verbose -Message "Successfully Created SIP Account"
        }
        catch
        {
            $message = "Unable to SIP Enable $Name on $SkypePool : " + $error[0].Exception
            Write-Warning -Message $message
        }
    }

    $EmailBody += "`n" + $_.DisplayName + "`t`t" + $_.Name + "`t" + $_.Country + "`t" + $_.SkypePool + "`t" + $_.MailboxDatabase + "`t" + $_.Type

    $iterator++
} # End of setting properties on successfully created room accounts


if ($createdRoomList.Count -gt 0)
{
    Write-Verbose -Message "Sending Summary Email"
    Send-MailMessage -SmtpServer $MailRelay -To $LogMailbox -From New-dvnConferenceRoom@dvn.com -Subject "New Conference Room Creation" -Body $EmailBody
}

} # End of New-dvnConferenceRoom function block

#===============================================================================

<# ***************************************************************
Name:		New-dvnDistributionGroup.ps1
Version:	1.0.01.07.2016
Author:     Jeff Brown

Version			Author
1.0.01.07.2016   Jeff Brown
    -Initial Code Creation

****************************************************************** #>

function New-dvnDistributionGroup()
{
<#
.SYNOPSIS
    Creates an Exchange distribution group and allows setting the type, managers,
    members, and custom email address for the group.

.PARAMETER DisplayName
    This will be the display name of the distribution group. It must start
    with "DL DVN", "DL USA", or "DL CDN".
    This parameter is mandatory at runtime.

.PARAMETER ManagedBy
    This is a list of the managers of the distribution group.
    This can be login names or email addresses.
    This parameter is optional at runtime.

.PARAMETER Members
    This is a list of the members of the distribution group (if not an automated group).
    This can be login names or email addresses.
    This parameter is optional at runtime.
    
.PARAMETER Type
    This is the type of distribution group.
    Options for Automated are: "Automated","AUTOMATED","automated","Auto","AUTO","auto","a","A"
    Options for Manual are: "Manual","MANUAL","manual","m","M"
    Options for Team are: "Team","TEAM","team","t","T"
    This parameter is mandatory at runtime.

.PARAMETER CustomEmail
    This allows adding a custom email address as an alias on the group.
    This parameter is optional at runtime.

.PARAMETER NoPrompt
    This is a switch parameter.
    When specified, it will not prompt to create the group.
    At minium the Display Name and Type will be needed before continuing.
    This parameter is optional at runtime.

.EXAMPLE
    New-dvnDistributionGroup -Display Name "DL Location Division, Department, Sub Department" -ManagedBy smithj1,john.smith@dvn.com -Members jonesd1,dan.jones@dvn.com -Type Manual
    This will create a manual distribution group with the specified display name, set the managers of the group, and add the members to the group.

.EXAMPLE
    New-dvnDistributionGroup -Display Name "DL Location Division, Department, Sub Department" -ManagedBy smithj1,john.smith@dvn.com -Type Automated -NoPrompt
    This will create an automated distribution group with the specified display name and sets the managers of the group without prompting for confirmation to make the group.

.EXAMPLE
    New-dvnDistributionGroup -Display Name "DL Location Division, Department, Sub Department" -ManagedBy smithj1,john.smith@dvn.com -Members jonesd1,dan.jones@dvn.com -Type Manual -CustomEmail myExample@dvn.com
    This will create a manual distribution group with the specified display name, set the managers of the group, and add the members to the group.
    It will also get an extra email address of "myExample@dvn.com".

.EXAMPLE
    Import-Csv DGsToCreate.csv | ForEach-Object {New-dvnDistributionGroup -DisplayName $_.DisplayName -Type $_.Type -ManagedBy $_.ManagedBy -Members $_.Members}
    This will import data from the DGsToCreate.csv file to create several distribution groups at once.
    The information file will need to have headers that match each parameter name.
    Names listed in the ManagedBy and Members column can be the login name or email address and separated by commas with no spaces.
    A sample file could look like this:
    
    DisplayName               Type        ManagedBy                       Members
    DL DVN Example One        Manual      user1,user2                     user3,user4,john.smith@dvn.com
    DL DVN EXample Two        Manual      user5,john.smith@dvn.com        user7,jane.smith@dvn.com

.NOTES
    This script requires a remote PowerShell session to Exchange as well as the appropriate RBAC
    permissions to create distribution groups.
#>

#*************************************************************************************
#************************       Parameters    ****************************************
#*************************************************************************************

[CmdletBinding()]
param(
    [Parameter(Position = 0, Mandatory = $true, ValueFromPipeLine = $true, ValueFromPipelineByPropertyName = $true,
        HelpMessage = "Enter the Display Name for the distribution group using the standard: DL Geo Location Division, Department, Sub Department")]
    [ValidateScript({
        if (($_.StartsWith("DL DVN ")) -eq $true -or ($_.StartsWith("DL USA ")) -eq $true -or $_.StartsWith("DL CDN "))
        {return $true}
        else
        {Throw "$_ : Invalid DL Naming Convention for $_. The start of the Display Name should be: 'DL DVN', 'DL USA', or 'DL CDN'."}
    })]
    [ValidateScript({
        if ((Get-DistributionGroup -Identity $_ -ErrorAction SilentlyContinue) -eq $null)
        {return $true}
        else
        {Throw "$_ already exists. Please check existing distribution groups"}
    })]
    [string]$DisplayName,

    [Parameter(Position = 1, ValueFromPipeLine = $true, ValueFromPipelineByPropertyName = $true)]
    [array]$ManagedBy = @(),

    [Parameter(Position = 2, ValueFromPipeLine = $true, ValueFromPipelineByPropertyName = $true)]
    [array]$Members = @(),

    [Parameter(Position = 3, Mandatory = $true, ValueFromPipeLine = $true, ValueFromPipelineByPropertyName = $true,
        HelpMessage = "Enter the Type of distribution group: Automated, Manual, or Team")]
    [ValidateSet("Automated","AUTOMATED","automated","Auto","AUTO","auto","a","A","Manual","MANUAL","manual","m","M","Team","TEAM","team","t","T")]
    [string]$Type,

    [Parameter(Position = 4, ValueFromPipeLine = $true, ValueFromPipelineByPropertyName = $true,
        HelpMessage = "Enter a custom email address ending with '@dvn.com'. This will be added as an alias on the distribution group.")]
    [ValidateScript({
        if ($_.EndsWith("@dvn.com") -eq $true)
        {return $true}
        else
        {Throw "$_ does NOT end with '@dvn.com' and is not a valid email address"}
    })]
    [string]$CustomEmail,

    [Parameter(Position = 5)]
    [switch]$NoPrompt
)

#*************************************************************************************
#***********************    Connection Check   ***************************************
#*************************************************************************************

# Verify PS Connection to Exchange, exits function if not connected
$connExchange = $false
$PSSessions = Get-PSSession
$PSSessions | ForEach-Object {
	if ($_.ConfigurationName -ilike "Microsoft.Exchange")
	{
		$connExchange = $true
	}
}

if ($connExchange -eq $false)
{
	Write-Error -Message "Current Remote PowerShell Session to Exchange NOT Detected -- Exiting"
	RETURN
}

#*************************************************************************************
#**************************     Variables     ****************************************
#*************************************************************************************

[string]$alias = $null
[string]$automatedOU = "corp.dvn.com/@Devon/exchange/distributionLists/automated"
[string]$manualOU = "corp.dvn.com/@Devon/exchange/distributionLists/manual"
[string]$teamsOU = "corp.dvn.com/@Devon/groups/function/teams"
[string]$OU = $null

$validMembers = New-Object System.Collections.ArrayList
$validManagedBy = New-Object System.Collections.ArrayList

[string]$ODCDataCenter = "oklahomaDataCenter"
[string]$Domain = "corp.dvn.com"
[string]$DomainController = (Get-ADDomainController -Discover -DomainName $Domain -SiteName $ODCDataCenter).HostName

[string]$LogMailbox = "ExchangeRunbookLogs@dvn.com"
[string]$MailRelay = "mailrelay.dvn.com"
[string]$Date = Get-Date
[string]$WhoAmI = $env:USERNAME

#*************************************************************************************
#**************************     Main Code     ****************************************
#*************************************************************************************

# Create $alias from $DisplayName by removing spaces and special characters
$alias = $DisplayName.Substring(3)
$alias = $alias.ToLower()
$alias = $alias -replace '[-&, ]','.'
$alias = $alias -replace '\.{2,}','.'   # Remove instances of two periods '..'

# Setting Organizational Unite in Active Directory based on $Type parameter value
switch ($Type)
{
    {$_ -in "Automated","AUTOMATED","automated","Auto","AUTO","auto","a","A"} {$OU = $automatedOU; break}
    {$_ -in "Manual","MANUAL","manual","m","M"} {$OU = $manualOU; break}
    {$_ -in "Team","TEAM","team","t","T"} {$OU = $teamsOU; break}
    Default {Write-Error "Unknown Distribution List Type - Exiting Script"; RETURN}
}

# Verifies list of ManagedBy for valid users if specified as a parameter
if ($ManagedBy.Count -ne 0)
{
    Write-Verbose -Message "Managed By Verification"
    $ManagedBy = $ManagedBy.Split(",")
    $ManagedBy | ForEach-Object {
        Write-Verbose -Message "Checking $_"
        $Valid = Get-Recipient -Identity $_ -ResultSize 2 -ErrorAction SilentlyContinue
        if ($Valid.Count -gt 1)
        {
            Write-Warning "Duplicate Users Found for $_"
        }
        elseif ($Valid -ne $null)
        {
            Write-Verbose "$_ Valid User for Managed By"
            $ValidManagedBy.Add((Get-Recipient -Identity $_).SamAccountName) | Out-Null
        }
        else
        {
            Write-Warning "$_ is not a valid user to add as a manager to the distribution group"
        }
    }
}

# Output distribution group information for viewing
$DGoutput = [PSCustomObject][ordered]@{
                'Display Name' = $DisplayName
                'Primary Email Address' = $alias + "@dvn.com"
                'Custom Email Address' = $CustomEmail
                'Managed By' = $validManagedBy
                'Type' = $Type
                'OU' = $OU
                'Created By' = $WhoAmI
                'Date' = $Date
            }

Write-Output $DGoutput

if ($NoPrompt -eq $false)
{
    $message = "`nDo you want to create the above Distribution List?"
    $yes = New-Object System.Management.Automation.Host.ChoiceDescription "&Yes", "Creates the distribution group"
    $no = New-Object System.Management.Automation.Host.ChoiceDescription "&No", "Does NOT create the request distribution group"
    $quit = New-Object System.Management.Automation.Host.ChoiceDescription "&Quit", "Exits the script without making any changes"
    $options = [System.Management.Automation.Host.ChoiceDescription[]]($yes, $no, $quit)
    $result = $host.ui.PromptForChoice($title, $message, $options, 0)

    if ($result -eq 0)
    {
        Write-Verbose -Message "Attempting to Create Distribution Group"
        try
        {
            if ($validManagedBy.Count -eq 0)
            {
                New-DistributionGroup -Name $alias -DisplayName $Displayname -OrganizationalUnit $OU -SamAccountName $alias -DomainController $DomainController -ErrorAction STOP | Out-Null
                Write-Verbose -Message "$DisplayName Created"
            }
            else
            {
                New-DistributionGroup -Name $alias -DisplayName $Displayname -OrganizationalUnit $OU -SamAccountName $alias -ManagedBy $validManagedBy -DomainController $DomainController -ErrorAction STOP | Out-Null
            }
            
            Send-MailMessage -SmtpServer $MailRelay -To $LogMailbox -From "New-dvnDistributionGroup@dvn.com" -Subject "New Distribution List Created" -Body ($DGoutput | Out-String)
        }
        catch
        {
            $message = "Distribution List Creation Failed" + $error[0].Exception
            Write-Error -Message $message
            RETURN
        }        
    }
    elseif ($result -eq 1 -or $result -eq 2)
    {
        Write-Verbose -Message "NOT Creating Distribution Group - Exiting Script`n"
        RETURN
    }
}
else
{
    # $NoPrompt should be $true and it will not prompt for creation approval
    Write-Verbose -Message "Attempting to Create Distribution Group"    
    try
    {
        if ($validManagedBy.Count -eq 0)
        {
            New-DistributionGroup -Name $alias -DisplayName $Displayname -OrganizationalUnit $OU -SamAccountName $alias -DomainController $DomainController -ErrorAction STOP | Out-Null
            Write-Verbose -Message "$DisplayName Created"
        }
        else
        {
            New-DistributionGroup -Name $alias -DisplayName $Displayname -OrganizationalUnit $OU -SamAccountName $alias -ManagedBy $validManagedBy -DomainController $DomainController -ErrorAction STOP  | Out-Null
        }
        
        Send-MailMessage -SmtpServer $MailRelay -To $LogMailbox -From "New-dvnDistributionGroup@dvn.com" -Subject "New Distribution List Created" -Body ($DGoutput | Out-String)
    }
    catch
    {
        $message = "Distribution List Creation Failed : " + $error[0].Exception
        Write-Error -Message $message 
        RETURN
    }
}

# Verify list of members for valid accounts before adding to distribution group
if ($Members.Count -ne 0)
{
    $Members = $Members.Split(",")
    Write-Verbose -Message "Member List Verification"
    $Members | ForEach-Object {
        Write-Verbose -Message "Checking $_"
        $Valid = Get-Recipient -Identity $_ -ResultSize 2 -ErrorAction SilentlyContinue
        if ($Valid.Count -gt 1)
        {
            Write-Warning "Duplicate Users Found for $_"
        }
        elseif ($Valid -ne $null)
        {
            try
            {
                Add-DistributionGroupMember -Identity $alias -Member $Valid.SamAccountName -DomainController $DomainController -ErrorAction STOP
                Write-Verbose "$_ Added to Distribution Group"
            }
            catch
            {
                $message = "Unable To Add $_ : " + $error[0].Exception
                Write-Warning -Message $message
            }
        }
        else
        {
            Write-Warning -Message "$_ is not a valid user to add to Distribution Group Membership"
        }
    }
} # End of verifying and adding members to the DL

# Adds a custom email address as an alias on the distribution group if it was specified as a parameter
if ($CustomEmail -ne '')
{
    Write-Verbose -Message "Attempting to Add Custom Email Address $CustomEmail"
    try
    {
        Set-DistributionGroup -Identity $alias -EmailAddresses @{add=$CustomEmail} -DomainController $DomainController -ErrorAction STOP
        Write-Verbose -Message "Custom Email Address Added"
    }
    catch
    {
        $message = "Unable to add custom email address $_ : " + $error[0].Exception
        Write-Warning -Message $message
    }
} # End of adding custom email address

} # End of New-dvnDistributionGroup Function

#===============================================================================

#Requires -Version 3

<# ***************************************************************
Name:		New-dvnSharedMailbox
Version:	20160606.21
Author:     Jeff Brown

Version			Author
1.0.12.29.2014  Jeff Brown
    -Initial Code Creation

1.1.02.12.2015  Jeff Brown
    -Added more user friendly prompts for $MailboxName, $Country,
    and $EmailAddress if not specified as parameters
    -Added requirement for PowerShell Version 3 for "{$_ -in 'Y','',$null}"
    in "switch {$CreateAnswer}"

1.2.05.18.2015  Jeff Brown
    -Removed option to create custom email address as our new hybrid
    Exchange environment does not populate additional email addresses
    when creating a mailbox without the email address policy enabled

1.3.06.16.2015  Jeff Brown
    -Added try/catch block when creating mailbox to catch existing
    mailboxes with the same name
    -Moved block of code that sets policies after the country switch
    as it was redundant being in both switch locations as they did
    the same thing

1.4.10.08.2015  Jeff Brown
    -Add check in $MailboxName parameter for '@' and 'dvn.com' as
    mailboxes were being named using the email address
    -Updated help block to remove references to creating custom
    email addresses

1.5.11.16.2015  Jeff Brown
    -Added removal of special characters when creating $Alias and
    $UserPrincipalName using RegEx (Thanks Steven Judd)

1.6.01.04.2016  Jeff Brown
    -Fixed issue where the script was setting the country for all
    mailboxes to 'CA' and not the correct country

2.0.01.11.2016  Jeff Brown
    -Rewrote script to condense code and clean it up, removed use of
    Write-Host and using Write-Verbose & Write-Error & Write-Warning,
    made parameters mandatory. Also fixed issue where $Alias was
    not being used in creation of mailbox so special characters
    were not being removed in the primary email address on creation.
    Still requires PowerShell Version 3 for line "{$_ -in 1, 2} {RETURN}".

20160606.21     Jeff Brown
    -Enabling ability to save Send on Behalf and Send As items to
    the shared mailbox option
    -Updated to new version tracking format

20170404.22     Mitch Shiever    
    -Changed the OU from shared to sharedExchangeOnline
    -Best practice is to create mailboxes online and migrate them 
     to Exchange Online.  We'll have a script that checks this OU
     for new shared mailboxes and it will auto migrate them.
    -Removed database servers that we are scheduled to decom
****************************************************************** #>

function New-dvnSharedMailbox()
{

<#
.SYNOPSIS
    Creates a shared mailbox in the Exchange environment

.PARAMETER DisplayName
    DisplayName is the display name of the mailbox.
    This name will also become the Alias and Email Address of the shared mailbox minus any special characters.
    Example: "Southern Business Unit Accounting"
    This parameter is mandatory at runtime.

.PARAMETER Country
    Country will create the mailbox on a US or Canadian database.
    Valid options are 'US' or 'CA'.
    This parameter is mandatory at runtime.

.EXAMPLE
    New-dvnSharedMailbox -DisplayName "My Shared Mailbox" -Country "US"
    This command will create a shared mailbox on a US database.

.EXAMPLE
    New-dvnSharedMailbox -DisplayName "My Shared Mailbox" -Country "CA"
    This command will create a shared mailbox on a Canadian database.

.NOTES
    -Requires PowerShell Version 3
    -Changed the OU where the shared mailbox gets created to automate the migration
     to Exchange Online.
#>

#*************************************************************************************
#************************       Parameters    ****************************************
#*************************************************************************************

[CmdletBinding()]
param(
	[Parameter(Position=0, Mandatory=$true, ValueFromPipeline=$true, ValueFromPipelinebyPropertyName=$true,
        HelpMessage="This is the user friendly display name of the shared mailbox, example: 'Southern Business Unit Accounting'.")]
    [ValidateScript({
        if ($_ -notmatch "@" -and $_ -notmatch "dvn.com") {return $true}
        else {Throw "INVALID DISPLAY NAME: $_ contains '@' or 'dvn.com'. The shared mailbox Display Name should not be the email address."}
    })]
    [ValidateScript({
        if ((Get-Mailbox -Identity $_) -eq $null) {return $true}
        else {Throw "A mailbox with this Display Name already exists. Please verify any existing shared mailboxes."}
    })]
    [string]$DisplayName,

	[Parameter(Position=1, Mandatory=$true, ValueFromPipeline=$true, ValueFromPipelinebyPropertyName=$true, HelpMessage="This is the country for the shared mailbox. Valid options are 'US' or 'CA'.")]
    [ValidateSet("US","CA")]
    [string]$Country,

    [Parameter(Position=2, ValueFromPipeline=$true)]
    [ValidateScript({
        if ((Get-Recipient -Identity $_) -eq $null) {return $true}
        else {Throw "An Exchange object with $_ already exists. Please verify any existing email address aliases."}
    })]
    [ValidateScript({
        if ($_.EndsWith("@dvn.com") -eq $true) {return $true}
        else {Throw "Invalid Email Address Format : $_ MUST contain '@dvn.com' at the end to be a valid email address."}
    })]
    [string]$CustomEmailAddress
)

#*************************************************************************************
#***********************    Connection Check   ***************************************
#*************************************************************************************

# Verify PS Connection to Exchange, exits function if not connected

$connExchange = $false
$PSSessions = Get-PSSession
$PSSessions | ForEach-Object {
	if ($_.ConfigurationName -ilike "Microsoft.Exchange")
	{
		$connExchange = $true
	}
}

if ($connExchange -eq $false)
{
	Write-Error -Message "Current Remote PowerShell Session to Exchange NOT Detected -- Exiting"
	RETURN
}

#*************************************************************************************
#**************************     Variables     ****************************************
#*************************************************************************************

[string]$SharedOU = "OU=sharedEO,OU=ResourceMailboxes,OU=Exchange,OU=@Devon,DC=Corp,DC=DVN,DC=com"

[string]$USDatabase = Get-Random -InputObject "EX220ADB01","EX220ADB02","EX220ADB03","EX220ADB04"
[string]$CADatabase = Get-Random -InputObject "EX230ADB01","EX230ADB02","EX230ADB03","EX230ADB04" 

[string]$Database = $null

[string]$RetentionPolicy = "Devon Shared Mailbox Retention Policy"

[string]$ODCDataCenter = "oklahomaDataCenter"
[string]$Domain = "corp.dvn.com"
[string]$DomainController = Get-ADDomainController -Discover -DomainName $Domain -SiteName $ODCDataCenter | select -ExpandProperty HostName

[string]$LogMailbox = "ExchangeRunbookLogs@dvn.com"
[string]$MailRelay = "mailrelay.dvn.com"
[string]$Date = Get-Date
[string]$WhoAmI = $env:USERNAME

# Builds $Alias by removing special characters and spaces from $DisplayName
[string]$Alias = $DisplayName -replace "[^a-zA-Z0-9]",""
[string]$UserPrincipalName = $Alias + "@dvn.com"

#*************************************************************************************
#**************************     Main Code     ****************************************
#*************************************************************************************

if ($Country -eq "CA")
{
    $Database = $CADatabase
}
elseif ($Country -eq "US")
{
    $Database = $USDatabase
}

$MailboxOutput = [PSCustomObject][ordered]@{
                'Display Name' = $DisplayName
                'Email Address' = $Alias + "@dvn.com"
                'Email Alias' = $CustomEmailAddress
                'Alias' = $Alias
                'UPN' = $UserPrincipalName
                'Database' = $Database
                'Domain Controller' = $DomainController
                'Created By' = $WhoAmI
                'Date' = $Date
            }

Write-Output $MailboxOutput

$message = "`nDo you want to create the above Shared Mailbox?"
$yes = New-Object System.Management.Automation.Host.ChoiceDescription "&Yes", "Creates the shared mailbox"
$no = New-Object System.Management.Automation.Host.ChoiceDescription "&No", "Does NOT create the shared mailbox and exits"
$quit = New-Object System.Management.Automation.Host.ChoiceDescription "&Quit", "Exits without making any changes"
$options = [System.Management.Automation.Host.ChoiceDescription[]]($yes, $no, $quit)
$answer = $host.ui.PromptForChoice($title, $message, $options, 0)

# $answer will contain an integer value: 0 for "Yes", 1 for "No", 2 for "Quit"

switch ($answer)
{
    0 {
        Write-Verbose -Message "Attempting to Create $DisplayName"
        try
        {
            New-Mailbox -Name $DisplayName -Alias $Alias -UserPrincipalName $UserPrincipalName -OrganizationalUnit $SharedOU -Database $Database -RetentionPolicy $RetentionPolicy -Shared -DomainController $DomainController -ErrorAction STOP | Out-Null
            Write-Verbose -Message "Mailbox Creation Command Executed"
        }
        catch
        {
            $message = "Unable to create mailbox : " + $error[0]
            Write-Error -Message $message
            RETURN
        }
                    
        Write-Verbose "Waiting for mailbox to be enabled"
        
        # Run loop until Get-Mailbox returns a $null value showing the mailbox has been created
        # If it does not show created after 20 loops or about 40 seconds, quits script with warning message
		[int]$Count = 0
        do
		{
            $MailboxExist = Get-Mailbox $DisplayName -DomainController $DomainController -ErrorAction SilentlyContinue
            $Count++
            if ($Count -gt 20)
            {
                Write-Warning -Message "Mailbox not found after creation. Please verify if it was created."
                RETURN
            }
			Start-Sleep -Seconds 2
		} while ($MailboxExist -eq $null)
        
        Write-Verbose -Message "Setting CAS Mailbox Properties"
        try
        {
            Set-CASMailbox -Identity $DisplayName -OwaMailboxPolicy "Default" -ActiveSyncEnabled:$false -DomainController $DomainController -ErrorAction STOP | Out-Null
            Write-Verbose -Message "CAS Mailbox Properties Configured Successfully"
        }
        catch
        {
            $message = "Unable to set CAS Mailbox Properties : " + $error[0]
            Write-Warning -Message $message
        }
        
        Write-Verbose -Message "Setting Country Attribute on User Account"
        try
        {
            Set-User -Identity $DisplayName -CountryOrRegion $Country -DomainController $DomainController -ErrorAction STOP | Out-Null
            Write-Verbose -Message "Country Attribute Configured Successfully"
        }
        catch
        {
            $message = "Unable to set Country on User Account : " + $error[0]
            Write-Warning -Message $message
        }

        Write-Verbose -Message "Setting Message Copy Options for Send on Behalf and Send As"
        try
        {
            Set-Mailbox -Identity $DisplayName -MessageCopyForSendOnBehalfEnabled $true -MessageCopyForSentAsEnabled $true -DomainController $DomainController -ErrorAction STOP | Out-Null
            Write-Verbose -Message "Messay Copy for Send on Behalf and Sent As Enabled Successfully"
        }
        catch
        {
            $message = "Unable to set Message Copy options : " + $error[0]
            Write-Warning -Message $message
        }

        if ($CustomEmailAddress -ne '')
        {
            Write-Verbose -Message "Setting Additional Email Address"
            try
            {
                Set-Mailbox -Identity $DisplayName -EmailAddresses @{add=$CustomEmailAddress} -DomainController $DomainController -ErrorAction STOP | Out-Null
                Write-Verbose -Message "Additional Email Address Added Successfully"
            }
            catch
            {
                $message = "Unable to Set Additional Email Address : " + $error[0]
                Write-Warning -Message $message
            }
        }
        
        Write-Verbose -Message "Sending Summary Email"
        try
        {
            Send-MailMessage -SmtpServer $MailRelay -To $LogMailbox -From NewSharedMailbox@dvn.com -Subject "New Shared Mailbox Created" -Body ($MailboxOutput | Out-String) -ErrorAction STOP
            Write-Verbose -Message "Summary Email Sent Successfully"
        }
        catch
        {
            $message = "Unable to Send Summary Email : " + $error[0]
            Write-Warning -Message $message
        }
    } # End of $answer equals 0 or "Yes"
    
    {$_ -in 1, 2} {RETURN} # End of $answer equals 1 (Do Not Create) or 2 (Quit Script)
    
    Default {
        Write-Warning -Message "Unknown option selected. Please rerun command."
        RETURN
    }
} # End of switch ($answer)

} # End of New-dvnSharedMailbox function

#===============================================================================

<# ***************************************************************
Name:		Set-dvnArchivePolicy.ps1
Version:	1.1.04.13.2015
Author:     Jeff Brown

Version			Author
1.0.01.26.2015  Jeff Brown
    -Initial Code Creation

1.1.04.13.2015  Jeff Brown
    -Added verification before changing archive policy

****************************************************************** #>


function Set-dvnArchivePolicy()
{

<#
.SYNOPSIS
    Changes the Archive Policy for a user

.PARAMETER MailboxName
    MailboxName is the name of the mailbox to change the Archive Policy
    This can either be the login name or email address of the account
    This parameter is mandatory

.PARAMETER ArchiveDays
    ArchiveDays is the number of days to change the Archive Policy
    Valid options are 90, 60, 30, 14
    This parameter is mandatory
	
.EXAMPLE
    Set-dvnArchivePolicy -MailboxName John.Doe@dvn.com -ArchiveDays 60
    This will change the Archive Policy for John.Doe@dvn.com to 60 days

.NOTES
    This function will verify if the 'DVN Never Move to Archive' is tagged on the
    Calendar, Tasks, Notes, and Journal folders. If it is not set, the function will
    end and prompt to have the Enable-dvnArchivePolicy script ran which can tag the
    folders. This requires impersonation rights and EWS DLL installed on the local computer
    and can only be ran by Exchange Administrators
#>

#*************************************************************************************
#************************       Parameters    ****************************************
#*************************************************************************************

[CmdletBinding()]
param(
	[Parameter(Position=0, Mandatory=$true, ValueFromPipeline=$true, ValueFromPipelinebyPropertyName=$true, HelpMessage="Name of mailbox to set archive policy (email address or alias)")]
    [string]$MailboxName,

    [Parameter(Position=1, Mandatory=$true, ValueFromPipeline=$true, ValueFromPipelinebyPropertyName=$true, HelpMessage="Valid options are 90, 60, 30, or 14")]
    [ValidateSet("90","60","30","14")]
    [string]$ArchiveDays
)

#*************************************************************************************
#***********************    Connection Check   ***************************************
#*************************************************************************************

# Verify PS Connection to Exchange before gathering variable data, exits function if not connected

$connExchange = $false
$PSSessions = Get-PSSession | select ConfigurationName,Availability
foreach ($PSSession in $PSSessions)
{
	If ($PSSession.ConfigurationName -ilike "Microsoft.Exchange")
	{
		$connExchange = $True
	}
}

If ($connExchange -eq $False)
{
	Write-Host "ERROR: Current Remote PowerShell Session to Exchange NOT Detected -- Exiting" -ForegroundColor White -BackgroundColor DarkRed
	RETURN
}

#*************************************************************************************
#**************************     Variables     ****************************************
#*************************************************************************************

[string]$RetentionPolicy   = $null
[array]$NeverArchives = @("Calendar","Tasks","Notes","Journal")

# Convert #ArchiveDays to corresponding Retention Policy Name
switch ($ArchiveDays)
{
    "90" {$RetentionPolicy = "Devon Default Archive and Retention Policy"; break}
    "60" {$RetentionPolicy = "Devon 60 Day Archive and Retention Policy"; break}
    "30" {$RetentionPolicy = "Devon 30 Day Archive and Retention Policy"; break}
    "14" {$RetentionPolicy = "Devon 14 Day Archive and Retention Policy"; break}
    Default {Write-Host "ERROR:" -ForegroundColor White -BackgroundColor DarkRed -NoNewline
            Write-Host "$ArchivePolicy is an invalid value. Exiting Function" -ForegroundColor Yellow
            RETURN}
} # End of switch ($ArchivePolicy)

#*************************************************************************************
#**************************     Main Code     ****************************************
#*************************************************************************************

# Verify Mailbox
Write-Host "`nChecking $MailboxName..." -NoNewline
$Valid = Get-Mailbox -Identity $MailboxName -ResultSize 3 -ErrorAction SilentlyContinue
if ($Valid.Count -gt 1)
{
    Write-Host "Duplicate Mailboxes Found" -ForegroundColor White -BackgroundColor DarkRed
    RETURN
}
elseif ($Valid.DistinguishedName -inotlike "*OU=people,OU=@Devon,DC=corp,DC=dvn,DC=com")
{
    Write-Host "NOT a Valid USER Mailbox" -ForegroundColor White -BackgroundColor DarkRed
    RETURN
}
elseif ($Valid.ArchiveDatabase -eq $null)
{
    Write-Host "Does NOT Have an Archive Mailbox - Provision One Before Continuing" -ForegroundColor White -BackgroundColor DarkRed
    RETURN
}
elseif ($Valid -ne $null)
{
    Write-Host "Valid`n" -ForegroundColor Green
}
else
{
    Write-Host "NOT Valid Mailbox" -ForegroundColor White -BackgroundColor DarkRed
    RETURN
} # End of mailbox verification

$CurrArchivePolicy = Get-Mailbox -Identity $MailboxName | select RetentionPolicy
if ($CurrArchivePolicy.RetentionPolicy -eq $RetentionPolicy)
{
    Write-Host "Archive Policy is already set to $ArchiveDays - No Changes Being Made`n" -ForegroundColor Yellow
    RETURN
}
else
{
    Write-Host "`nCurrent Archive Policy:`t" -NoNewLine
    Write-Host $CurrArchivePolicy.RetentionPolicy "`n" -ForegroundColor Yellow
    Write-Host "New Archive Policy:`t`t" -NoNewLine
    Write-Host $RetentionPolicy -ForegroundColor Yellow

    [string]$ChangeAnswer = "noanswer"
    do
    {
        Write-Host "`nConfirm Archive Policy Change?"
	    Write-Host "[Y] Yes    " -f yellow -NoNewLine; Write-Host "[N] No    [Q] Quit Script (default is 'Y'): " -NoNewLine; $ChangeAnswer = Read-Host
        $ChangeAnswer = $ChangeAnswer.ToUpper()
    
        switch ($ChangeAnswer)
        {
            {$_ -in 'Y','',$null} # $ChangeAnswer is empty/null/yes
                {
                    Write-Host "`nPreparing to change Archive Policy to $ArchiveDays" -ForegroundColor Yellow
    
                    foreach ($NeverArchive in $NeverArchives)
                    {
                        Write-Host "Verifying 'DVN Never Move to Archive' is set on $NeverArchive folder: " -NoNewline
                        $result = Get-MailboxFolderStatistics -Identity $MailboxName -FolderScope $NeverArchive | select ArchivePolicy
                        if ($result.ArchivePolicy -ne "DVN Never Move to Archive")
                        {
                            Write-Host "FAIL" -ForegroundColor White -BackgroundColor DarkRed
                            Write-Host "`n$NeverArchive" -ForegroundColor Yellow -NoNewline
                            Write-Host " folder does not have 'DVN Never Move to Archive' tag applied"
                            Write-Host "Have Messaging Team run Enable-dvnArchivePolicy to correct this"
                            Write-Host "No Changes Being Made - Exiting Script`n" -ForegroundColor Yellow
                            RETURN
                        }
                        else
                        {
                            Write-Host "Pass" -ForegroundColor Green
                        }
                    } # End of foreach ($NeverArchive in $NeverArchives)

                    Set-Mailbox -Identity $MailboxName -RetentionPolicy $RetentionPolicy -ErrorAction SilentlyContinue | Out-Null
                    Write-Host "Verifying change..." -NoNewline
    
                    do
	                {
		                $NewRetentionPolicy = Get-Mailbox $MailboxName -ErrorAction SilentlyContinue | select -ExpandProperty RetentionPolicy
                        $Count++
                        if ($Count -gt 20)
                        {
                            Write-Host "ERROR:" -ForegroundColor White -BackgroundColor DarkRed -NoNewline
                            Write-Host " Retention Policy was not set. Please run again - Exiting Script" -ForegroundColor Yellow
                            RETURN
                        }
		                Start-Sleep -Seconds 2
	                } while ($NewRetentionPolicy -ne $RetentionPolicy)

                    Write-Host "Done" -ForegroundColor Green
                    Write-Host "`nExecuting Start-ManagedFolderAssistant to apply changes immediately`n"
                    Start-ManagedFolderAssistant $MailboxName
                } # End of Y, blank, or NULL
            {$_ -in 'N','Q'} # $ChangeAnswer is no/quit script
                {Write-Host "`nArchive policy NOT being changed. Please run command again. Exiting script!" -ForegroundColor Yellow; RETURN}
        
            Default # $ChangeAnswer is not a valid option
                {Write-Host "`n$_ is an unknown option. Please try again using a valid option." -ForegroundColor Yellow; $ChangeAnswer = "noanswer"; break}

            } # End of switch ($ChangeAnswer)
        } while ($ChangeAnswer -eq "noanswer")

} # End of if($CurrArchivePolicy -eq $RetentionPolicy)
} # End of Set-dvnArchivePolicy Function

#===============================================================================

<# ***************************************************************
Name:		Set-dvnMailboxPermission
Version:	1.1.02.18.2015
Author:     Jeff Brown

Version			Author
1.0.01.07.2015  Jeff Brown
    -Initial code creation

1.1.02.18.2015  Jeff Brown
    -Switched from adding mailbox name to the distinguished name
    of the mailbox when validating the mailbox into the $ValidMailbox
    array. This prevents multiple mailboxes being found when adding
    two or more have the same name but are in different OUs. Also
    changed the section where permissions are added/removed to
    use the distinguished name but show the display name for
    readability
****************************************************************** #>


function Set-dvnMailboxPermission()
{

<#
.SYNOPSIS
    Add or removes Full Access and Send As permissions on shared mailboxes

.PARAMETER Mailboxes
    Mailboxes is the list of shared mailboxes whose permissions need to be changed
    Multiple mailboxes can be entered at once when they are separated by commas using the alias or email address
    For example:  sharedMailbox1,sharemailbox2@dvn.com

.PARAMETER Users
    Users is the list of users who will have permissions added or removed from the shared mailboxes
	Multiple users can be entered at once when they are separated by commas using their alias or email address
    For example:  smithj1,jane.smith@dvn.com

.PARAMETER AddFullAccess
    AddFullAccess will add Full Access permissions to the list of shared mailboxes for the list of users
    This parameter can only be paired with AddSendAs and cannot be mixed with a Remove parameter

.PARAMETER AddSendAs
    AddSendAs will add Send As permissions to the list of shared mailboxes for the list of users
    This parameter can only be paired with AddFullAccess and cannot be mixed with a Remove parameter

.PARAMETER RemoveFullAccess
    RemoveFullAccess will remove Full Access permissions to the list of shared mailboxes for the list of users
    This parameter can only be paired with RemoveSendAs and cannot be mixed with an Add parameter

.PARAMETER RemoveSendAs
    RemoveSendAs will remove Send As permissions to the list of shared mailboxes for the list of users
    This parameter can only be paired with RemoveFullAccess and cannot be mixed with an Add parameter

.EXAMPLE
    Set-dvnMailboxPermission -AddFullAccess
    This command will prompt for the users and shared mailboxes to add Full Access

.EXAMPLE
    Set-dvnMailboxPermission -Mailboxes mysharedmailbox1 -Users smithj10 -AddFullAccess -AddSendAs
    This command will grant smithj10 Full Access and Send As permissions on the mysharedmailbox1 Shared Mailbox

.EXAMPLE
    Set-dvnMailboxPermission -Mailboxes mysharedmailbox1@dvn.com -RemoveFullAccess
    This command will prompt for the list of users to remove Full Access permissions on mysharedmailbox1@dvn.com

.NOTES
    Parameters AddFullAccess and AddSendAs are a parameter set along with RemoveFullAccess and RemoveSendAs
    are a parameter set. This prevents trying to AddFullAccess and RemoveFullAccess at the same time.

#>

#*************************************************************************************
#************************       Parameters    ****************************************
#*************************************************************************************

[CmdletBinding()]
param(
	[Parameter(Position=0, Mandatory=$false,ValueFromPipeline=$true,ValueFromPipelinebyPropertyName=$true)]
    [array]$Mailboxes = @(),

	[Parameter(Position=1, Mandatory=$false,ValueFromPipeline=$true,ValueFromPipelinebyPropertyName=$true)]
    [array]$Users = @(),

    [Parameter(Position=2, ParameterSetName='Add')]
    [switch]$AddFullAccess,

    [Parameter(Position=3, ParameterSetName='Add')]
    [switch]$AddSendAs,

    [Parameter(Position=4, ParameterSetName='Remove')]
    [switch]$RemoveFullAccess,

    [Parameter(Position=5, ParameterSetName='Remove')]
    [switch]$RemoveSendAs
)

#*************************************************************************************
#***********************    Connection Check   ***************************************
#*************************************************************************************

# Verify PS Connection to Exchange, exits function if not connected

$connExchange = $false
$PSSessions = Get-PSSession | select ConfigurationName,Availability
foreach ($PSSession in $PSSessions)
{
	If ($PSSession.ConfigurationName -ilike "Microsoft.Exchange")
	{
		$connExchange = $True
	}
}

If ($connExchange -eq $False)
{
	Write-Host "ERROR: Current Remote PowerShell Session to Exchange NOT Detected -- Exiting" -ForegroundColor White -BackgroundColor DarkRed
	RETURN
}

#*************************************************************************************
#**************************     Variables     ****************************************
#*************************************************************************************

$ValidUsers = New-Object System.Collections.ArrayList
$ValidMailboxes = New-Object System.Collections.ArrayList

[string]$LogMailbox = "ExchangeRunbookLogs@dvn.com"
[string]$MailRelay = "mailrelay.dvn.com"
[string]$Date = Get-Date
[string]$WhoAmI = whoami
[string]$EmailInformation = $null

#*************************************************************************************
#**************************     Main Code     ****************************************
#*************************************************************************************

# Get user and date for summary email
$EmailInformation += "Changes made by`t`t: $WhoAmI"
$EmailInformation += "`nDate`t`t`t`t: $Date`n"

# Gets list of mailboxes
if ($Mailboxes.Count -eq 0)
{
    Write-Host "`nShared Mailboxes" -ForegroundColor Yellow
    Write-Host "Enter mailboxes to add/remove user permissions"
    Write-host "Separate multiple mailboxes with commas and no spaces"
    Write-Host "Mailbox name can be alias or email address"
    Write-Host "For example: SharedMailbox,SharedMailbox@dvn.com"    
    $Mailboxes = Read-Host
    $Mailboxes = $Mailboxes.Split(",")
}

# Performs mailbox validation
Write-Host "`nMailbox List Verification" -ForegroundColor Yellow
foreach ($Mailbox in $Mailboxes)
{
    Write-Host "Checking $Mailbox..." -NoNewline
    $Valid = Get-Mailbox -Identity $Mailbox -ResultSize 3 -ErrorAction SilentlyContinue
    if ($Valid.Count -gt 1)
    {
        Write-Host "Duplicate Mailboxes Found" -ForegroundColor White -BackgroundColor DarkRed
    }
    elseif ($Valid.DistinguishedName -inotlike "*OU=shared,OU=resourceMailboxes,OU=exchange,OU=@Devon,DC=corp,DC=dvn,DC=com" -and $Valid.DistinguishedName -inotlike "*OU=voiceMail,OU=resourceMailboxes,OU=exchange,OU=@Devon,DC=corp,DC=dvn,DC=com")
    {
        Write-Host "NOT a Shared Mailbox" -ForegroundColor White -BackgroundColor DarkRed
    }
    elseif ($Valid -ne $null)
    {
        Write-Host "Valid" -ForegroundColor Green
        $MailboxName = Get-Mailbox -Identity $Mailbox | Select -ExpandProperty DistinguishedName
        $ValidMailboxes.Add($MailboxName) | Out-Null
    }
    else
    {
        Write-Host "NOT Valid Mailbox" -ForegroundColor White -BackgroundColor DarkRed
    }
}

# Prepares list of users to set mailbox permissions for
if ($Users.Count -eq 0)
{
    Write-Host "`nUsers to Add" -ForegroundColor Yellow
    Write-Host "Enter user(s) to add/remove mailbox permissions"
    Write-host "Separate multiple users with commas and no spaces"
    Write-Host "User can be login names or email addresses"
    Write-Host "For example: doej1,John.Doe@dvn.com"    
    $Users = Read-Host
    $Users = $Users.Split(",")
}

# Performs user validation
Write-Host "`nUser List Verification" -ForegroundColor Yellow
foreach ($User in $Users)
{
    Write-Host "Checking $User..." -NoNewline
    $Valid = Get-Recipient -Identity $User -ResultSize 3 -ErrorAction SilentlyContinue
    if ($Valid.Count -gt 1)
    {
        Write-Host "Duplicate Users Found" -ForegroundColor White -BackgroundColor DarkRed
    }
    elseif ($Valid -ne $null)
    {
        Write-Host "Valid" -ForegroundColor Green
        $SamAccountName = Get-Recipient -Identity $User | Select -ExpandProperty SamAccountName
        $ValidUsers.Add($SamAccountName) | Out-Null
    }
    else
    {
        Write-Host "NOT Valid User" -ForegroundColor White -BackgroundColor DarkRed
    }
}

# Blank line
Write-Host

# Add or removes permissions based on parameters specified only if there is at least 1 mailbox and 1 user
if ($ValidUsers.Count -ge 1 -and $ValidMailboxes.Count -ge 1)
{
    if ($AddFullAccess -eq $true)
    {
        foreach ($Mailbox in $ValidMailboxes)
        {
            foreach ($User in $ValidUsers)
            {
                Add-MailboxPermission -Identity $Mailbox -User $User -AccessRights FullAccess | Out-Null
                $DisplayName = Get-Mailbox -Identity $Mailbox | Select-Object -ExpandProperty Name
                Write-Host "Full Access granted to $User on $DisplayName"
                $EmailInformation += "`n`nFull Access granted to $User on $DisplayName"
            }
        }
    }

    if ($AddSendAs -eq $true)
    {
        foreach ($Mailbox in $ValidMailboxes)
        {
            foreach ($User in $ValidUsers)
            {
                Add-ADPermission -Identity $Mailbox -User $User -ExtendedRights Send-As | Out-Null
                $DisplayName = Get-Mailbox -Identity $Mailbox | Select-Object -ExpandProperty Name
                Write-Host "Send As granted to $User on $DisplayName"
                $EmailInformation += "`n`nSend As granted to $User on $DisplayName"
            }
        }
    }

    if ($RemoveFullAccess -eq $true)
    {
        foreach ($Mailbox in $ValidMailboxes)
        {
            foreach ($User in $ValidUsers)
            {
                Remove-MailboxPermission -Identity $Mailbox -User $User -AccessRights FullAccess -Confirm:$false | Out-Null
                $DisplayName = Get-Mailbox -Identity $Mailbox | Select-Object -ExpandProperty Name
                Write-Host "Full Access removed for $User on $DisplayName"
                $EmailInformation += "`n`nFull Access removed for $User on $DisplayName"
            }
        }
    }

    if ($RemoveSendAs -eq $true)
    {
        foreach ($Mailbox in $ValidMailboxes)
        {
            foreach ($User in $ValidUsers)
            {
                Remove-ADPermission -Identity $Mailbox -User $User -ExtendedRights Send-As -Confirm:$false | Out-Null
                $DisplayName = Get-Mailbox -Identity $Mailbox | Select-Object -ExpandProperty Name
                Write-Host "Send As removed for $User on $DisplayName"
                $EmailInformation += "`n`nSend As removed for $User on $DisplayName"
            }
        }
    }
    # Send summary email
    Send-MailMessage -SmtpServer $MailRelay -To $LogMailbox -From SetMailboxPermission@dvn.com -Subject "Shared Mailbox Permission Changes" -Body $EmailInformation
}
else
{
    Write-Host "Not enough users or mailboxes specified. At least one user and one mailbox should be valid. Please try again. Exiting script...`n" -ForegroundColor Yellow
    RETURN
}

Write-Host

} # End of Set-dvnMailboxPermission function

#===============================================================================

<# ***************************************************************
Name:		Set-dvnOnlineMailboxPermission
Version:	201601010.1
Author:     Jeff Brown

Version			Author
201610101.1     Jeff Brown
****************************************************************** #>


function Set-dvnOnlineMailboxPermission()
{

<#
.SYNOPSIS
    Add or removes Full Access and Send As permissions on Exchange Online shared mailboxes

.PARAMETER Mailboxes
    Mailboxes is the list of shared mailboxes whose permissions need to be changed
    Multiple mailboxes can be entered when separated by commas using the alias or email address
    For example:  sharedMailbox1,sharemailbox2@dvn.com

.PARAMETER Users
    Users is the list of users who will have permissions added or removed from the shared mailboxes
	Multiple users can be entered when they are separated by commas using their alias or email address
    For example:  smithj1,jane.smith@dvn.com

.PARAMETER AddFullAccess
    AddFullAccess will add Full Access permissions to the list of shared mailboxes for the list of users
    This parameter can only be paired with AddSendAs and cannot be mixed with a Remove parameter

.PARAMETER AddSendAs
    AddSendAs will add Send As permissions to the list of shared mailboxes for the list of users
    This parameter can only be paired with AddFullAccess and cannot be mixed with a Remove parameter

.PARAMETER RemoveFullAccess
    RemoveFullAccess will remove Full Access permissions to the list of shared mailboxes for the list of users
    This parameter can only be paired with RemoveSendAs and cannot be mixed with an Add parameter

.PARAMETER RemoveSendAs
    RemoveSendAs will remove Send As permissions to the list of shared mailboxes for the list of users
    This parameter can only be paired with RemoveFullAccess and cannot be mixed with an Add parameter

.EXAMPLE
    Set-dvnOnlineMailboxPermission -Mailboxes mysharedmailbox1,SharedMailbox2@dvn.com -Users smithj10 -AddFullAccess -AddSendAs
    This command will grant smithj10 Full Access and Send As permissions on the mysharedmailbox1 and SharedMailbox2@dvn.com Shared Mailboxes

.EXAMPLE
    Set-dvnOnlineMailboxPermission -Mailboxes mysharedmailbox1@dvn.com -Users john.smith@dvn.com -RemoveFullAccess
    This command will prompt for the list of users to remove Full Access permissions on mysharedmailbox1@dvn.com

.NOTES
    Parameters AddFullAccess and AddSendAs are a parameter set along with RemoveFullAccess and RemoveSendAs
    are a parameter set. This prevents trying to perform Add and Remove commands at the same time.

#>

#*************************************************************************************
#************************       Parameters    ****************************************
#*************************************************************************************

[CmdletBinding()]
param(
	[Parameter(Mandatory=$true, HelpMessage="This is a list of shared mailboxes to modify the permissions of.")]
    [array]$Mailboxes = @(),

	[Parameter(Mandatory=$true, HelpMessage="This is a list of users to give or to remove permissions for.")]
    [array]$Users = @(),

    [Parameter(ParameterSetName='Add')]
    [switch]$AddFullAccess,

    [Parameter(ParameterSetName='Add')]
    [switch]$AddSendAs,

    [Parameter(ParameterSetName='Remove')]
    [switch]$RemoveFullAccess,

    [Parameter(ParameterSetName='Remove')]
    [switch]$RemoveSendAs
)

#*************************************************************************************
#***********************    Connection Check   ***************************************
#*************************************************************************************

# Verify PS Connection to Exchange Online, exits if not connected

$PSSessions = (Get-PSSession).ComputerName
if ($PSSessions -notcontains "outlook.office365.com")
{
    throw "Current Remote PowerShell Session to Exchange Online NOT Detected -- Exiting"
}

#*************************************************************************************
#**************************     Variables     ****************************************
#*************************************************************************************

$ValidUsers = New-Object System.Collections.ArrayList
$ValidMailboxes = New-Object System.Collections.ArrayList

[string]$LogMailbox = "ExchangeRunbookLogs@dvn.com"
[string]$MailRelay = "mailrelay.dvn.com"
[string]$emailFrom = "Set-dvnOnlineMailboxPermission@dvn.com"
[string]$emailSubject = "Shared Mailbox Permissions Changes"
[string]$emailBody = $null

#*************************************************************************************
#**************************     Main Code     ****************************************
#*************************************************************************************

# Get user and date for summary email
$emailBody += "Changes made by: $env:USERNAME"
$emailBody += "`nDate: $(Get-Date)"

# Performs mailbox validation
Write-Verbose "Mailbox List Verification"
foreach ($Mailbox in $Mailboxes)
{
    Write-Verbose "Verifying $Mailbox as a valid shared mailbox"
    $Valid = Get-Mailbox -Identity $Mailbox -ResultSize 3 -ErrorAction SilentlyContinue
    if ($Valid.Count -gt 1)
    {
        Write-Warning "Duplicate Mailboxes Found for $Mailbox"
    }
    elseif ($Valid.RecipientTypeDetails -ne "SharedMailbox")
    {
        Write-Warning "$Mailbox is not of SharedMailbox Recipient Type"
    }
    elseif ($Valid -ne $null)
    {
        Write-Verbose "$Mailbox is a valid shared mailbox"
        $ValidMailboxes.Add($Valid) | Out-Null
    }
    else
    {
        Write-Warning "$Mailbox is NOT a valid mailbox"
    }
}

# Performs user validation
Write-Verbose "Verifying User List"
foreach ($User in $Users)
{
    Write-Verbose "Checking $User as a valid user"
    $Valid = Get-User -Identity $User -ResultSize 3 -ErrorAction SilentlyContinue
    if ($Valid.Count -gt 1)
    {
        Write-Warning "$User resulted in uplicate users found" -ForegroundColor White -BackgroundColor DarkRed
    }
    elseif ($Valid -ne $null)
    {
        Write-Verbose "$User is a valid user"
        $ValidUsers.Add($Valid) | Out-Null
    }
    else
    {
        Write-Warning "$User is not Valid User"
    }
}

# Add or removes permissions based on parameters specified only if there is at least 1 mailbox and 1 user
if ($ValidUsers.Count -ge 1 -and $ValidMailboxes.Count -ge 1)
{
    if ($AddFullAccess -eq $true)
    {
        foreach ($Mailbox in $ValidMailboxes)
        {
            foreach ($User in $ValidUsers)
            {
                try
                {
                    Add-MailboxPermission -Identity $Mailbox.Identity -User $User.Identity -AccessRights FullAccess -ErrorAction STOP | Out-Null
                    $output = [PSCustomObject][ordered]@{
                        Mailbox = $($Mailbox.DisplayName)
                        User = $($User.DisplayName)
                        Action = "Add Full Access"
                    }
                    Write-Output $output
                    $emailBody += "`n`nFull Access granted to $User on $DisplayName"
                }
                catch
                {
                    Write-Warning "Unable to add Full Access permissions to $($Mailbox.DisplayName) for $($User.UserPrincipalName) : $_"
                }
            }
        }
    }
    
    if ($AddSendAs -eq $true)
    {
        foreach ($Mailbox in $ValidMailboxes)
        {
            foreach ($User in $ValidUsers)
            {
                try
                {
                    Add-RecipientPermission -Identity $Mailbox.Identity -Trustee $User.Identity -AccessRights SendAs -Confirm:$false -ErrorAction STOP | Out-Null
                    $output = [PSCustomObject][ordered]@{
                        Mailbox = $($Mailbox.DisplayName)
                        User = $($User.DisplayName)
                        Action = "Add Send As Access"
                    }
                    Write-Output $output
                    $emailBody += "`n`nSend As granted to $($User.UserPrincipalName) on $($Mailbox.DisplayName)"                    
                }
                catch
                {
                    Write-Warning "Unable to add Send As permissions to $($Mailbox.DisplayName) for $($User.UserPrincipalName) : $_"
                }
            }
        }
    }

    if ($RemoveFullAccess -eq $true)
    {
        foreach ($Mailbox in $ValidMailboxes)
        {
            foreach ($User in $ValidUsers)
            {
                try
                {
                    Remove-MailboxPermission -Identity $Mailbox.Identity -User $User.Identity -AccessRights FullAccess -Confirm:$false -ErrorAction STOP | Out-Null
                    $output = [PSCustomObject][ordered]@{
                        Mailbox = $($Mailbox.DisplayName)
                        User = $($User.DisplayName)
                        Action = "Removed Full Access"
                    }
                    Write-Output $output
                    $emailBody += "`n`nFull Access removed for $($User.UserPrincipalName) on $($Mailbox.DisplayName)"                    
                }
                catch
                {
                    Write-Warning "Unable to remove Full Access permissions to $($Mailbox.DisplayName) for $($User.UserPrincipalName) : $_"
                }
            }
        }
    }

    if ($RemoveSendAs -eq $true)
    {
        foreach ($Mailbox in $ValidMailboxes)
        {
            foreach ($User in $ValidUsers)
            {
                try
                {
                    Remove-RecipientPermission -Identity $Mailbox.Identity -Trustee $User.Identity -AccessRights SendAs -Confirm:$false -ErrorAction STOP | Out-Null
                    $output = [PSCustomObject][ordered]@{
                        Mailbox = $($Mailbox.DisplayName)
                        User = $($User.DisplayName)
                        Action = "Removed Send As"
                    }
                    Write-Output $output
                    $emailBody += "`n`nSend As removed for $($User.UserPrincipalName) on $($Mailbox.DisplayName)"
                }
                catch
                {
                    Write-Warning "Unable to remove Send As permissions to $($Mailbox.DisplayName) for $($User.UserPrincipalName) : $_"
                }
            }
        }
    }
    
    # Send summary email
    Send-MailMessage -SmtpServer $MailRelay -To $LogMailbox -From $emailFrom -Subject $emailSubject -Body $emailBody
}
else
{
    Write-Warning "Not enough users or mailboxes specified. At least one user and one mailbox should be valid. Please try again."
    RETURN
}

} # End of Set-dvnOnlineMailboxPermission function

#===============================================================================

<# ***************************************************************
Name:		Test-dvnExchangeHealth
Version:	1.3.08.08.2015
Author:     Jeff Brown

Version			Author
1.0.01.07.2015  Jeff Brown
    -Initial code creation

1.0.01.15.2015  Jeff Brown
    -Update to new Devon Function naming convention
    -Changed PS Exchange connection test to not try and establish one
    if it's not available

1.1.04.14.2015  Jeff Brown
    -Removed option to test individual servers
    -Updated testing database copies to use hash tables for output
    -Added functionality to test OWA page response
    -Added functionality to test Edge server connectivity
    -Reduced variables for specific Exchange server types

1.2.05.19.2015  Jeff Brown
    -Updated OWA checking to check all virtual directories testing
    the presence of the healthcheck.htm file for each virtual
    directory

1.3.08.08.2015  Jeff Brown
    -Fixed issue with RelayTest portion of script not working.
    Looks like I had deleted the [array]$Recipients variable
****************************************************************** #>

Function Test-dvnExchangeHealth()
{

<#
.SYNOPSIS
	Tests the health of the Exchange environment and the mail relay (if parameter specified)

.PARAMETER RelayTest
	If parameter is included, it will send emails through the mail relay to the recipients specified
	This paramter is optional

.EXAMPLE
	Test-dvnExchangeHealth
	This command will run checks against the entire Exchange environment

.EXAMPLE
	Test-dvnExchangeHealth -RelayTest
	This command will run checks against the entire Exchange environment and the mail relay
    Recipient email addresses for mail relay testing will be prompted during execution
		
.NOTES
	
#>

#*************************************************************************************
#*****************************    Parameters   ***************************************
#*************************************************************************************

[CmdletBinding()]
param(
	[Parameter(Position=0, Mandatory=$false)]
	[switch]$RelayTest
)

#*************************************************************************************
#***********************    Connection Check   ***************************************
#*************************************************************************************

# Verify PS Connection to Exchange before gathering variable data, exits function if not connected

$connExchange = $false
$PSSessions = Get-PSSession | select ConfigurationName,Availability
foreach ($PSSession in $PSSessions)
{
	If ($PSSession.ConfigurationName -ilike "Microsoft.Exchange")
	{
		$connExchange = $True
	}
}

If ($connExchange -eq $False)
{
	Write-Host "ERROR: Current Remote PowerShell Session to Exchange NOT Detected -- Exiting" -ForegroundColor White -BackgroundColor DarkRed
	RETURN
}

#*************************************************************************************
#******************************    Variables   ***************************************
#*************************************************************************************

[array]$RelayServers		= @("ODCMSMXEX200P","ODCMSMXEX201P","ODCMSMXEX202P","mailrelay.dvn.com")
[array]$Recipients          = @()
[array]$ExchangeServers     = @(Get-ExchangeServer | Sort-Object Name)
[array]$ActiveDatabases     = @(Get-MailboxDatabase | Where-Object {$_.Recovery -eq $false})
[array]$DAGServers          = @((Get-DatabaseAvailabilityGroup).Servers | Sort-Object)
[array]$VirtualDirs         = @("owa","ecp","oab","rpc","ews","mapi","Microsoft-Server-ActiveSync","Autodiscover")

#*************************************************************************************
#******************************    Main Code   ***************************************
#*************************************************************************************

Write-Host "`n*** Checking Health Sets ***`n" -ForegroundColor Yellow

foreach ($ExchangeServer in $ExchangeServers)
{
    if (($ExchangeServer.IsClientAccessServer -eq $true) -or ($ExchangeServer.IsMailboxServer -eq $true))
    {
	    Write-Host "`n$ExchangeServer : " -NoNewLine
	    $health = @(Get-HealthReport -Server $ExchangeServer.Name | where {$_.AlertValue -eq "Unhealthy"})
	    $healthOutput = $health.HealthSet -join ", "
		
	    if ($health.Count -gt 0)
	    {
		    Write-Host "Not Healthy" -ForegroundColor White -BackgroundColor DarkRed
		    Write-Host "Health Set(s) in Unhealthy State: " -NoNewLine
		    Write-Host $healthOutput -ForegroundColor Cyan
	    }
	    else
	    {
		    Write-Host "Health Sets Healthy" -ForegroundColor Green
	    }
    }
} # End of Checking Health Sets

Write-Host "`n*** Checking Services ***`n" -ForegroundColor Yellow

foreach ($ExchangeServer in $ExchangeServers)
{
	# Checks Exchange services are running on server
	# Test-ServiceHealth only works for Mailbox or Mailbox/Client Access dual role in 2013
	# Client Access Servers are not currently checked for services running
	$servicesGood = $True
	$badServices = @()
				
	If ($ExchangeServer.IsMailboxServer -eq $true)
	{		
		$services = Test-ServiceHealth -Server $ExchangeServer.Name
		Write-Host $ExchangeServer" : " -NoNewLine
			
		foreach ($service in $services)
		{
			if ($service.RequiredServicesRunning -eq $false)
			{
				$servicesGood = $False
				$badServices += $service.ServicesNotRunning
			}
		}
		
		# Removes any duplicated services in array
		$badServices = $badServices | Select-Object -Unique
			
		if ($servicesGood -eq $true)
		{
			Write-Host "All Services Running" -ForegroundColor green
		}
		else
		{
			Write-Host "Services Not Running" -ForegroundColor White -BackgroundColor DarkRed
			Write-Host "Following Services are Not Running:"
			foreach ($badService in $badServices)
			{
				Write-Host $badService
			}
		}
	}
} # End of Checking Service Health

Write-host "`n*** Checking Edge Availability ***`n" -ForegroundColor Yellow
foreach ($ExchangeServer in $ExchangeServers)
{
    if ($ExchangeServer.IsEdgeServer -eq $true)
    {
        Write-Host $ExchangeServer.Name": " -NoNewLine
        if ((Test-Connection -ComputerName $ExchangeServer.Fqdn -Count 2 -Quiet) -eq $true)
        {
            Write-Host "Available" -ForegroundColor Green
        }
        else
        {
            Write-Host "Not Available" -ForegroundColor Red
        }
    }
} # End of Checking Edge Availability

Write-host "`n*** Checking Virtual Directories ***" -ForegroundColor Yellow
foreach ($ExchangeServer in $ExchangeServers)
{
    if ($ExchangeServer.IsClientAccessServer -eq $true)
    {
        Write-Host "`n"$ExchangeServer.Name -ForegroundColor Yellow
        foreach ($VirtualDir in $VirtualDirs)
        {
            try
            {
                [string]$url = "https://" + $ExchangeServer.Fqdn + "/" + $VirtualDir + "/healthcheck.htm"
                Write-Host "`t$url : " -NoNewline
                $webCheck = Invoke-WebRequest $url -ErrorAction STOP
                if (($webCheck.StatusCode -ne "200") -or ($webCheck.StatusDescription -ne "OK"))
                {
                    Write-Host "Error" -ForegroundColor White -BackgroundColor DarkRed
                }
                else
                {
                    Write-Host "Healthy" -ForegroundColor Green
                }
            }
            catch
            {
                Write-Host "Error:" $_.Exception.Message -ForegroundColor White -BackgroundColor DarkRed
            }
        }
    }
} # End of checking virtual directory health

Write-Host "`n*** Checking DAG Health ***`n" -ForegroundColor Yellow		
foreach ($DAGServer in $DAGServers)
{
    [bool]$replHealth = $true
    [array]$badTests = @()
	Write-Host $DAGServer" : " -NoNewline
	$ReplTests = Test-ReplicationHealth -Server $DAGServer
	foreach ($ReplTest in $ReplTests)
	{
		If ($ReplTest.Result -ne "Passed")
		{
			$replHealth = $false
			$obj = New-Object PSObject
			$obj | Add-Member -MemberType NoteProperty -Name "DAG Test" -Value $ReplTest.Check
			$obj | Add-Member -MemberType NoteProperty -Name "Error" -Value $ReplTest.Error
			$badTests += $obj
		}
	}
	if ($replHealth -eq $true)
	{
		Write-Host "Replication Tests Passed" -ForegroundColor Green
	}
	else
	{
		Write-Host "Replication Tests Failed" -ForegroundColor White -BackgroundColor DarkRed
		Write-Host "Following Tests Did Not Pass:"
		$badTests | Format-Table -AutoSize			 	
	}		
}

Write-Host "`n*** Checking Database Copy Status ***`n" -ForegroundColor Yellow

foreach ($ActiveDatabase in $ActiveDatabases)
{
    [array]$copies = @(Get-MailboxDatabaseCopyStatus -Identity $ActiveDatabase.Name)
    foreach ($copy in $copies)
    {
        $badCopy = $null
        if (($copy.Status -ne "Mounted") -and ($copy.Status -ne "Healthy"))
        {
            $badCopy = [ordered]@{
                Name = $copy.Name
                Status = $copy.Status
                CopyQueue = $copy.CopyQueueLength
                ReplayQueue = $copy.ReplayQueueLength
                ContentIndex = $copy.ContentIndexState
            }
        } # End of if ($copy.Status -ne "Mounted")
        elseif ($copy.ContentIndexState -ne "Healthy")
        {
            $badCopy = [ordered]@{
                Name = $copy.Name
                Status = $copy.Status
                CopyQueue = $copy.CopyQueueLength
                ReplayQueue = $copy.ReplayQueueLength
                ContentIndex = $copy.ContentIndexState
            }
        } # End of elseif ($copy.ContentIndexState -ne "Healthy")
        elseif ($copy.CopyQueueLength -ge 100)
        {
           $badCopy = [ordered]@{
                Name = $copy.Name
                Status = $copy.Status
                CopyQueue = $copy.CopyQueueLength
                ReplayQueue = $copy.ReplayQueueLength
                ContentIndex = $copy.ContentIndexState
            }
        } # End of elseif ($copy.CopyQueueLength -ge 100)
        elseif ($copy.ReplayQueueLength -ge 100)
        {
           $badCopy = [ordered]@{
                Name = $copy.Name
                Status = $copy.Status
                CopyQueue = $copy.CopyQueueLength
                ReplayQueue = $copy.ReplayQueueLength
                ContentIndex = $copy.ContentIndexState
            }
        } # End of elseif ($copy.ReplayQueueLength -ge 100)
        
        if ($badCopy -ne $null)
        {
            $outputObj = New-Object -TypeName PSObject -Property $badCopy
            Write-Output $outputObj
        }
    } # End of foreach ($copy in $copies)
} # End of foreach ($ActiveDatabase in $ActiveDatabases)

# If $RelayTest parameter is specified, test each mail relay server and VIP
if ($RelayTest -eq $true)
{
	Write-Host "`n*** Testing Mail Relay ***`n" -ForegroundColor Yellow	
	do
	{
		$addRecipient = Read-Host "Enter email address to test relay (email@domain.com)(leave blank to continue)"
		if ($addRecipient -ne "")
		{
			# Testing for valid email input (email@domain.com)
			if ($addRecipient -match "\w+\@+\w+\.+\w")
			{
				$Recipients += $addRecipient
			}
			else
			{
				Write-Host "Invalid email format, please try again"
			}
		}
	} until ($addRecipient -eq "")
			
	foreach ($RelayServer in $RelayServers)
	{
		foreach ($Recipient in $Recipients)
		{
			Write-Host "Testing Mail Relay Server $RelayServer"
			Send-MailMessage -SmtpServer $RelayServer -To $Recipient -From "ExchangeHealthScript@dvn.com" -Subject "$RelayServer Mail Relay Test" -Body "This is a test through mail relay server $RelayServer"
		}
	}
} # End of RelayTest

} # End of Test-dvnExchangeHealth function

#===============================================================================

<# ***************************************************************
Name:		Test-dvnMailRelay
Version:	20160727.2
Author:     Jeff Brown

Version			Author
1.0.02.29.2016  Jeff Brown
    -Initial code creation

20160727.2     Jeff Brown
    -Updated to new version standard
    -Added parameter RelaysToUse in order to test individual servers,
    the relay name, or both
    -Added CGY relay servers to $RelayServer for testing

****************************************************************** #>


function Test-dvnMailRelay()
{

<#
.SYNOPSIS
    Sends email messages through each server configured for open relay as well as
    the availability of the mailrelay.dvn.com name.

.PARAMETER Recipients
    This parameters is an array of strings that holds email addresses to send test
    email messages to. More than one value can be entered as long as they are separated by commas
    and no spaces. The email addresses can be internal Devon addresses or outside email addresses.

    For example:

    -Recipients john.doe@dvn.com,jane@externaldomain.com

.PARAMETER RelaysToUse
    This parameter specifies whether or not to test each individual server in the relay,
    test the relay name, or test all. Valid options for this parameter are 'individual',
    'relay', or 'all'.
	
.EXAMPLE
    Test-dvnMailRelay -Recipients john.doe@dvn.com -RelaysToUse all
    This command will send a test email to john.doe@dvn.com using the individual servers
    as well as the mailrelay.dvn.com name.

.EXAMPLE
    Test-dvnMailRelay -Recipients john.doe@dvn.com,jane@externaldomain.com -RelaysToUse relay
    This command will send to john.doe@dvn.com and to the external recipient jane@externaldomain.com
    using only the mailrelay.dvn.com name.    
#>

#*************************************************************************************
#************************       Parameters    ****************************************
#*************************************************************************************

[CmdletBinding()]
param(
    [Parameter(Position = 0, Mandatory = $true, HelpMessage = "Enter one or more email addresses separated by commas to send test messages to. Example: john.doe@dvn.com,jane.doe@externaldomain.com")]
    [string[]]$Recipients = @(),

    [Parameter(Position = 1, Mandatory = $true, HelpMessage = "Enter the test mode: 'individual' (tests each server in the relay), 'relay' (tests mailrelay.dvn.com), or 'all'")]
    [ValidateSet("individual", "relay", "all" )]
    [string]$RelaysToUse 
)


#*************************************************************************************
#**************************     Variables     ****************************************
#*************************************************************************************

[array]$RelayServers = @("ODCMSMXEX200P","ODCMSMXEX201P","ODCMSMXEX202P","CGYMSMXEX200P","CGYMSMXEX201P","CGYMSMXEX202P")
[string]$RelayName = "mailrelay.dvn.com"


#*************************************************************************************
#**************************     Functions     ****************************************
#*************************************************************************************

function Send-RelayMail ($server, $to)
{
    try
    {
        Write-Verbose "Testing Mail Relay Server $server"
		Send-MailMessage -SmtpServer $server -To $to -From "Test-dvnMailRelay@dvn.com" -Subject "$server Mail Relay Test" -Body "This is a test through mail relay server $server" -ErrorAction STOP
    }
    catch
    {
        $message = "Problem Sending Email Through Relay Server $server : " + $error[0].Exception
        Write-Warning -Message $message
    }
}

#*************************************************************************************
#**************************     Main Code     ****************************************
#*************************************************************************************

switch ($RelaysToUse)
{
    'individual'
    {
        foreach ($RelayServer in $RelayServers)
        {
	        foreach ($Recipient in $Recipients)
	        {
		        Send-RelayMail $RelayServer $Recipient
	        }
        }
    }

    'relay'
    {
        foreach ($Recipient in $Recipients)
        {
            Send-RelayMail $RelayName $Recipient
        }
    }

    'all'
    {
        $RelayServers += $RelayName
        foreach ($RelayServer in $RelayServers)
        {
	        foreach ($Recipient in $Recipients)
	        {
		        Send-RelayMail $RelayServer $Recipient
	        }
        }
    }

} # End of switch($RelaysToUse)


} # End of Test-dvnMailRelay Function

#===============================================================================

